/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/beans/archive/AudioArchiveBean.java,v 1.1.1.1 2001/09/29 20:38:49 jonathan Exp $
 * $Log: AudioArchiveBean.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.beans.archive;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.ServletRequest;
import org.opendj.beans.onair.OnAirBean;
import org.opendj.util.Util;
import org.opendj.util.ProcessUtil;
import org.opendj.util.PLSConstants;
import org.opendj.util.DBUtil;
import org.opendj.util.DBFormat;
import org.opendj.util.Config;

public class AudioArchiveBean {

    public static final int SORT_LISTENERS_DESC = 0;
    public static final int SORT_LISTENERS_ASC  = 1;
    public static final int SORT_TIME_DESC      = 2;
    public static final int SORT_TIME_ASC       = 3;

    public static final String[] SORT_TEXT =
    { "numlisteners DESC", "numlisteners", "starttime DESC", "starttime" };

    // XXX hard-coded things here
    private static String ARCHIVE_PATH   = null;
    private static String PLS_DIR        = null;
    private static String PLS_LINK       = null;
    public static final String ARCHIVE_URL    = Util.EMPTY_STRING;
    public static final String FILE_EXTENSION = ".mp3";

    private String itsChannelPath = null;

    private int     itsArchid        = -1;
    private int     itsBitrate       = -1;
    private int     itsDjid          = -1;
    private String  itsDjname        = null;
    private String  itsShowname      = null;
    private int     itsGenre         = -1;
    private String  itsGenreval      = null;
    private int     itsDjmood        = -1;
    private String  itsMoodval       = null;
    private long    itsStarttime     = -1;
    private long    itsActualstart   = -1;
    private long    itsEndtime       = -1;
    private long    itsLastaired     = -1;
    private int     itsChannelid     = -1;
    private int     itsNumlisteners  = -1;
    private int     itsNumcomments   = -1;
    private int     itsRconntotal    = -1;
    private int     itsRconncount    = -1;
    private int     itsRconnavg      = -1;
    private int     itsRaudiototal   = -1;
    private int     itsRaudiocount   = -1;
    private int     itsRaudioavg     = -1;
    private int     itsRstyletotal   = -1;
    private int     itsRstylecount   = -1;
    private int     itsRstyleavg     = -1;
    private boolean itsIscompleted   = false;
    private boolean itsIskeeper      = false;
    private boolean itsIsrerunnable  = false;
    private int     itsKarmaearned   = -1;
    private String  itsShowfile      = null;
    private String  itsShowurl       = null;
    private String  itsTranscripturl = null;
    private String  itsDjcomments    = null;

    public AudioArchiveBean () {
        if ( ARCHIVE_PATH == null ) {
            ARCHIVE_PATH = Config.global().getProperty(Config.PROP_ARCHIVEDIR);
        }
        PLS_DIR = Config.global().getProperty(Config.PROP_PLSDIR);
        PLS_LINK = Config.global().getProperty(Config.PROP_PLSLINK);
    }

    public int getArchid () { return itsArchid; }
    public void setArchid ( int theArchid ) { itsArchid = theArchid; }

    public int getBitrate () { return itsBitrate; }
    public void setBitrate ( int theBitrate ) throws SQLException { 
        if ( itsBitrate != theBitrate ) {
            itsBitrate = theBitrate;
            writeBitrate();
        }
    }

    public int getDjid () { return itsDjid; }
    public void setDjid ( int theDjid ) { itsDjid = theDjid; }

    public String getDjname () { return itsDjname; }
    public void setDjname ( String theDjname ) { itsDjname = theDjname; }

    public String getShowname () { return itsShowname; }
    public void setShowname ( String theShowname ) { itsShowname = theShowname; }

    public int getGenre () { return itsGenre; }
    public void setGenre ( int theGenre ) { itsGenre = theGenre; }

    public String getGenreval () { return itsGenreval; }
    public void setGenreval ( String theGenreval ) { itsGenreval = theGenreval; }

    public int getDjmood () { return itsDjmood; }
    public void setDjmood ( int theDjmood ) { itsDjmood = theDjmood; }

    public String getMoodval () { return itsMoodval; }
    public void setMoodval ( String theMoodval ) { itsMoodval = theMoodval; }

    public long getStarttime () { return itsStarttime; }
    public void setStarttime ( long theStarttime ) { itsStarttime = theStarttime; }

    public long getActualstart () { return itsActualstart; }
    public void setActualstart ( long theActualstart ) { itsActualstart = theActualstart; }

    public long getEndtime () { return itsEndtime; }
    public void setEndtime ( long theEndtime ) { itsEndtime = theEndtime; }

    public long getLastaired () { return itsLastaired; }
    public void setLastaired ( long theLastaired ) { itsLastaired = theLastaired; }

    public int getNumlisteners () { return itsNumlisteners; }
    public void setNumlisteners ( int theNumlisteners ) { itsNumlisteners = theNumlisteners; }

    public int getChannelid () { return itsChannelid; }
    public void setChannelid ( int theChannelid ) { itsChannelid = theChannelid; }

    public int getNumcomments () { return itsNumcomments; }
    public void setNumcomments ( int theNumcomments ) { itsNumcomments = theNumcomments; }

    public int getRconntotal () { return itsRconntotal; }
    public void setRconntotal ( int theRconntotal ) { itsRconntotal = theRconntotal; }

    public int getRconncount () { return itsRconncount; }
    public void setRconncount ( int theRconncount ) { itsRconncount = theRconncount; }

    public int getRconnavg () { return itsRconnavg; }
    public void setRconnavg ( int theRconnavg ) { itsRconnavg = theRconnavg; }

    public int getRaudiototal () { return itsRaudiototal; }
    public void setRaudiototal ( int theRaudiototal ) { itsRaudiototal = theRaudiototal; }

    public int getRaudiocount () { return itsRaudiocount; }
    public void setRaudiocount ( int theRaudiocount ) { itsRaudiocount = theRaudiocount; }

    public int getRaudioavg () { return itsRaudioavg; }
    public void setRaudioavg ( int theRaudioavg ) { itsRaudioavg = theRaudioavg; }

    public int getRstyletotal () { return itsRstyletotal; }
    public void setRstyletotal ( int theRstyletotal ) { itsRstyletotal = theRstyletotal; }

    public int getRstylecount () { return itsRstylecount; }
    public void setRstylecount ( int theRstylecount ) { itsRstylecount = theRstylecount; }

    public int getRstyleavg () { return itsRstyleavg; }
    public void setRstyleavg ( int theRstyleavg ) { itsRstyleavg = theRstyleavg; }

    public boolean getIscompleted () { return itsIscompleted; }
    public void setIscompleted ( boolean theIscompleted ) { itsIscompleted = theIscompleted; }

    public boolean getIskeeper () { return itsIskeeper; }
    public void setIskeeper ( boolean theIskeeper ) { itsIskeeper = theIskeeper; }

    public boolean getIsrerunnable () { return itsIsrerunnable; }
    public void setIsrerunnable ( boolean theIsrerunnable ) { itsIsrerunnable = theIsrerunnable; }

    public int getKarmaearned () { return itsKarmaearned; }
    public void setKarmaearned ( int theKarmaearned ) { itsKarmaearned = theKarmaearned; }

    public String getShowfile () { return itsShowfile; }
    public void setShowfile ( String theShowfile ) { itsShowfile = theShowfile; }

    public String getShowurl () { return itsShowurl; }
    public void setShowurl ( String theShowurl ) { itsShowurl = theShowurl; }

    public String getTranscripturl () { return itsTranscripturl; }
    public void setTranscripturl ( String theTranscripturl ) { itsTranscripturl = theTranscripturl; }

    public String getDjcomments () { return itsDjcomments; }
    public void setDjcomments ( String theDjcomments ) { itsDjcomments = theDjcomments; }

    public String getStarttimestring () { 
        return DBFormat.FormatExtYearDateTime(new Date(itsStarttime));
    }

    public long getDuration () {
        return itsEndtime - itsActualstart;
    }
    public long getScheduledDuration () {
        return itsEndtime - itsStarttime;
    }
    public String getDurationstring () {
        return DBFormat.FormatDuration(itsEndtime - itsActualstart);
    }

    public void init ( ResultSet theRS ) throws SQLException {
        setArchid          (theRS.getInt   ("archid"));
        setBitrate         (theRS.getInt   ("bitrate"));
        setDjid            (theRS.getInt   ("djid"));
        setDjname          (theRS.getString("djname"));
        setShowname        (theRS.getString("showname"));
        setGenre           (theRS.getInt   ("genre"));
        setGenreval        (theRS.getString("genreval"));
        setDjmood          (theRS.getInt   ("djmood"));
        setMoodval         (theRS.getString("moodval"));
        setStarttime       (theRS.getLong  ("starttime"));
        setActualstart     (theRS.getLong  ("actualstart"));
        setEndtime         (theRS.getLong  ("endtime"));
        setLastaired       (theRS.getLong  ("lastaired"));
        setNumlisteners    (theRS.getInt   ("numlisteners"));
        setChannelid       (theRS.getInt   ("channelid"));
        setNumcomments     (theRS.getInt   ("numcomments"));
        setRconntotal      (theRS.getInt   ("rconntotal"));
        setRconncount      (theRS.getInt   ("rconncount"));
        setRconnavg        (theRS.getInt   ("rconnavg"));
        setRaudiototal     (theRS.getInt   ("raudiototal"));
        setRaudiocount     (theRS.getInt   ("raudiocount"));
        setRaudioavg       (theRS.getInt   ("raudioavg"));
        setRstyletotal     (theRS.getInt   ("rstyletotal"));
        setRstylecount     (theRS.getInt   ("rstylecount"));
        setRstyleavg       (theRS.getInt   ("rstyleavg"));
        setIscompleted     (theRS.getInt   ("iscompleted") == 1);
        setIskeeper        (theRS.getInt   ("iskeeper") == 1);
        setIsrerunnable    (theRS.getInt   ("isrerunnable") == 1);
        setKarmaearned     (theRS.getInt   ("karmaearned"));
        setShowfile        (theRS.getString("showfile"));
        setShowurl         (theRS.getString("showurl"));
        setTranscripturl   (theRS.getString("transcripturl"));
        setDjcomments      (theRS.getString("djcomments"));
    }

    public boolean lookupByArchID ( Connection theConn ) throws SQLException {
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL = "SELECT * FROM ArchivedShow WHERE archid = ? ";
        try {
            aPS = theConn.prepareStatement(aSQL);
            aPS.setInt(1, getArchid());
            aRS = aPS.executeQuery();
            if ( aRS.next() ) {
                init(aRS);
                return true;
            }
            return false;

        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
        }
    }

    public boolean lookupByDJandTime ( Connection theConn ) throws SQLException {
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL
            = "SELECT * "
            + "FROM ArchivedShow "
            + "WHERE djid = ? "
            + "AND starttime = ? "
            + "AND endtime = ? "
            ;
        try {
            aPS = theConn.prepareStatement(aSQL);
            aPS.setInt (1, getDjid());
            aPS.setLong(2, getStarttime());
            aPS.setLong(3, getEndtime());
            aRS = aPS.executeQuery();
            if ( aRS.next() ) {
                init(aRS);
                return true;
            }
            return false;

        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
        }
    }
    
    public void startArchive ( String theCommand, 
                               OnAirBean theOnAirBean ) throws IOException {

        itsChannelPath = theOnAirBean.getchannelpath();
        String anArchiveResult = null;

        // Construct the filename so it includes the show name,
        // the channel name, and the date, but also so it guarantees uniqueness
        // by including the current millisecond time.
        String aDateString
            = DBFormat.FormatNormalDate(new Date())
            + " " + String.valueOf(System.currentTimeMillis())
            ;
        String aBaseFile = Util.makeSafeFileName(theOnAirBean.getshowname()
                                                 + " " + itsChannelPath
                                                 + " " + aDateString);
        aBaseFile += FILE_EXTENSION;
        
        // Prepend the archive path to the filename
        itsShowfile = ARCHIVE_PATH + File.separator + aBaseFile;

        // Create a showurl for this guy..
        itsShowurl = ARCHIVE_URL + "/" + aBaseFile;

        anArchiveResult = ProcessUtil.execStringCommand(theCommand + " " +
                                                        itsShowfile + " " + 
                                                        itsChannelPath);
    }

    public void stopArchive ( String theCommand ) throws IOException {
        String anArchiveResult = null;
        anArchiveResult = ProcessUtil.execStringCommand(theCommand + " " +
                                                        itsChannelPath);
    }

    public static String getServerPrefix() {
        return PLS_LINK;
    }

    public void createPLS () throws IOException {
        File aPLSFile = null;
        PrintWriter aWriter = null;
        try {
            aPLSFile = new File(PLS_DIR + File.separator
                                + "archive" + getArchid() + ".pls");
            if ( aPLSFile.exists() ) {
                if ( !aPLSFile.delete() ) {
                    throw new IOException("Archive file: " + aPLSFile.getAbsolutePath() 
                                          + " could not be deleted!");
                }
            }
            aWriter = new PrintWriter(new FileWriter(aPLSFile));
            aWriter.println(PLSConstants.PLS_PREFIX);
            aWriter.println("File1=" + PLS_LINK + getShowurl());
            aWriter.println(PLSConstants.PLS_HELP_TEXT);

        } finally {
            try { aWriter.close(); } catch ( Exception e ) {}
        }
    }

    private void writeBitrate () throws SQLException {
        Connection aConn = null;
        PreparedStatement aPS = null;
        String aSQL 
            = "UPDATE ArchivedShow SET bitrate = ? WHERE archid = ? ";
        try {
            aConn = DBUtil.getConnection();
            aPS = aConn.prepareStatement(aSQL);
            aPS.setInt(1, itsBitrate);
            aPS.setInt(2, itsArchid);
            aPS.executeUpdate();

        } finally {
            DBUtil.closeStatement(aPS);
            DBUtil.closeConnection(aConn);
        }
    }

    public String toString () {
        return "[AudioArchive djid=" + itsDjid
            + ", djname='" + itsDjname + "'"
            + ", starttime=" + DBFormat.FormatDateTimeEx(itsStarttime)
            + ", endtime=" + DBFormat.FormatDateTimeEx(itsEndtime)
            + ", karmaearned=" + itsKarmaearned
            + "]"
            ;
    }
}
