/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/beans/channel/ChannelBean.java,v 1.2 2001/09/29 22:47:19 jonathan Exp $
 * $Log: ChannelBean.java,v $
 * Revision 1.2  2001/09/29 22:47:19  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.beans.channel;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.opendj.util.Util;
import org.opendj.util.Config;
import org.opendj.util.PLSConstants;
import org.opendj.util.DBUtil;

public class ChannelBean {

    public static final String PLAYLIST_EXTENSION = ".pls";

    private int     itsCID           = -1;
    private String  itsName          = null;
    private String  itsPath          = null;
    private boolean itsIsreal           = true;
    private String  itsDescription   = null;
    private String  itsDjDescription = null;

    public ChannelBean () {}

    public int    getcid() { return itsCID; }
    public void   setcid(int theCID) { itsCID = theCID; }

    public String getname() { return itsName; }
    public void   setname(String theName) { itsName = theName; }

    public String getjsname() { return Util.makeSafeFileName(itsName); }

    public String getplname() { 
        return Util.makeSafeFileName(itsPath.substring(1)) + PLAYLIST_EXTENSION; 
    }

    public boolean getisreal() { return itsIsreal; }
    public void    setisreal(boolean theIsreal) { itsIsreal = theIsreal; }

    public String getpath() { return itsPath; }
    public void   setpath(String thePath) { itsPath = thePath; }
    public String getroomname() { 
        int aSpacePos = itsName.indexOf(" ");
        if ( aSpacePos == -1 ) return itsName;
        return itsName.substring(0, aSpacePos).trim();
    }

    public String getdescription() { return itsDescription; }
    public void   setdescription(String theDescription) { itsDescription = theDescription; }

    public String getdjdescription() { return itsDjDescription; }
    public void   setdjdescription(String theDjDescription) { itsDjDescription = theDjDescription; }

    public void init ( ResultSet theRS ) throws SQLException {

        itsCID         = theRS.getInt   ("cid");
        itsName        = theRS.getString("name");
        itsIsreal      = (theRS.getInt("isreal") == 1);
        itsPath        = theRS.getString("path");
        itsDescription = theRS.getString("description");
        itsDjDescription = theRS.getString("djdescription");
    }

    public void lookup (Connection theConn) throws SQLException {

        String aSQL
            = "SELECT cid, name, path, isreal, description, djdescription "
            + "FROM Channel "
            + "WHERE cid = ? "
            ;
        PreparedStatement aPS = null;
        ResultSet aRS = null;

        try {
            aPS   = theConn.prepareStatement(aSQL);
            aPS.setInt(1, itsCID);
            aRS   = aPS.executeQuery();
                    
            if ( aRS.next() ) {
                init( aRS );
            }

        } finally {
            DBUtil.closeStatement(aPS);
            DBUtil.closeRS(aRS);
        }
    }

    public void save (Connection theConn) throws SQLException {
        
        boolean isNewChannel = (getcid() == -1);
        String aSQL = null;
        PreparedStatement aPS = null;

        if ( isNewChannel ) {
            aSQL = "INSERT INTO Channel ( name, path, isreal, description, djdescription ) VALUES ( ?, ?, ?, ? ) ";
        } else {
            aSQL = "UPDATE Channel SET name = ?, path = ?, isreal = ?, description = ?, djdescription = ? WHERE cid = ? ";
        } 

        try {
            aPS   = theConn.prepareStatement(aSQL);

            int i=1;
            aPS.setString(i++, getname());
            aPS.setString(i++, getpath());
            aPS.setInt   (i++, getisreal() ? 1 : 0);
            aPS.setString(i++, getdescription());
            aPS.setString(i++, getdjdescription());
            if ( !isNewChannel ) {
                aPS.setInt(i++, getcid());
            }
            
            aPS.executeUpdate();
            
        } finally {
            DBUtil.closeStatement(aPS);
        }
    }

    /**
     * Generate a PLS file for the live stream
     */
    public void createPLS () throws IOException {

        // Read config data
        String LIVE_PLS_LINK = Config.global().getProperty(Config.PROP_PLSLIVELINKPREFIX);
        String LIVE_PLS_DIR = Config.global().getProperty(Config.PROP_PLSLIVEDIR);

        File aPLSFile = new File(LIVE_PLS_DIR + File.separator + getplname());
        if ( aPLSFile.exists() ) {
            if ( !aPLSFile.delete() ) {
                throw new IOException("PLS file for live stream: " 
                                      + aPLSFile.getAbsolutePath() 
                                      + " could not be deleted!");
            }
        }
        PrintWriter aWriter = null;
        try {
            aWriter = new PrintWriter(new FileWriter(aPLSFile));
            aWriter.println(PLSConstants.PLS_PREFIX);
            aWriter.println("File1=" + LIVE_PLS_LINK + getpath());
            aWriter.println(PLSConstants.PLS_HELP_TEXT);
        } finally {
            try { aWriter.close(); } catch ( Exception e ) {}
        }
    }

    public boolean equals ( Object o ) {
        if ( o instanceof ChannelBean ) {
            return ( ((ChannelBean) o).getcid() == getcid() );
        }
        return false;
    }
}
