/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/beans/dj/DjBean.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: DjBean.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.beans.dj;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.StringTokenizer;
import java.util.Map;
import java.util.HashMap;
import java.util.Date;
import javax.mail.internet.InternetAddress;
import org.opendj.util.Util;
import org.opendj.util.DBUtil;
import org.opendj.util.DBFormat;
import org.opendj.util.DuplicateException;
import org.opendj.util.Mailer;
import org.opendj.util.CryptoUtil;
import org.opendj.util.debug;

public class DjBean extends org.opendj.share.DjBean {

    public static final int LOGIN_OK          = 0;
    public static final int LOGIN_NOSUCHUSER  = 1;
    public static final int LOGIN_BADPASSWORD = 2;

    protected String itsOldEmail = null;
    protected boolean isEmailChanged = false;
    protected String itsNewPassword = null;

    public DjBean () {}

    public void   setemail(String theEmail) { 
        if ( itsEmail != null && 
             theEmail != null &&
             itsEmail.equals(theEmail) ) {
            isEmailChanged = false;
        } else {
            itsOldEmail = itsEmail;
            isEmailChanged = true;
            itsNewPassword = CryptoUtil.getRandomWord(6);
            setpassword(itsNewPassword);
        }
        itsEmail = theEmail; 
    }

    public void   setpassword(String thePassword) { 
        itsPassword = Util.getMD5Base64(thePassword);
    }


    public boolean getisemailchanged() { return isEmailChanged; }

    public void init ( ResultSet rs ) throws SQLException {
        setdjid           (rs.getInt   ("djid"));
        setipaddr         (rs.getString("ipaddr"));
        itsEmail =         rs.getString("email");
        itsPassword =      rs.getString("password");
        setshowname       (rs.getString("showname"));
        setdjname         (rs.getString("djname"));
        setgenre          (rs.getInt   ("genre"));
        setgenreval       (rs.getString("gname"));
        setjoindate       (rs.getLong  ("joindate"));
        setfirstbroadcast (rs.getLong  ("firstbroadcast"));
        setnumbroadcasts  (rs.getInt   ("numbroadcasts"));
        setnumreruns      (rs.getInt   ("numreruns"));
        settotallisteners (rs.getInt   ("totallisteners"));
        setnumsubs        (rs.getInt   ("numsubs"));
        setnumstartlate   (rs.getInt   ("numstartlate"));
        setmaxweeklyblocks(rs.getInt   ("maxweeklyblocks"));
        setmaxsingleblocks(rs.getInt   ("maxsingleblocks"));
        setkarma          (rs.getLong  ("karma"));
        setnumendearly    (rs.getInt   ("numendearly"));
        setnumnoshow      (rs.getInt   ("numnoshow"));
        setcity           (rs.getString("city"));
        setstate          (rs.getString("state"));
        setcountry        (rs.getString("country"));
        setphone          (rs.getString("phone"));
        setmusicdesc      (rs.getString("musicdesc"));
        setwebpage        (rs.getString("webpage"));
    }

    public void populateMap ( Map m ) {
        m.put("djid", new Integer(getdjid()));
        m.put("ipaddr", getipaddr());
        m.put("email", getemail());
        m.put("password", getpassword());
        m.put("showname", getshowname());
        m.put("djname", getdjname());
        m.put("genre", new Integer(getgenre()));
        m.put("gname", getgenreval());
        m.put("joindate", new Long(getjoindate()));
        m.put("firstbroadcast", new Long(getfirstbroadcast()));
        m.put("numbroadcasts", new Integer(getnumbroadcasts()));
        m.put("numreruns", new Integer(getnumreruns()));
        m.put("totallisteners", new Integer(gettotallisteners()));
        m.put("numsubs", new Integer(getnumsubs()));
        m.put("numstartlate", new Integer(getnumstartlate()));
        m.put("maxweeklyblocks", new Integer(getmaxweeklyblocks()));
        m.put("maxsingleblocks", new Integer(getmaxsingleblocks()));
        m.put("karma", new Long(getkarma()));
        m.put("numendearly", new Integer(getnumendearly()));
        m.put("numnoshow", new Integer(getnumnoshow()));
        m.put("city", getcity());
        m.put("state", getstate());
        m.put("country", getcountry());
        m.put("phone", getphone());
        m.put("musicdesc", getmusicdesc());
        m.put("webpage", getwebpage());
    }

    public void save (Connection theConn) throws SQLException, DuplicateException {

        String aSQL = null;
        boolean isNewDj = (getdjid() == -1);
        PreparedStatement ps = null;
        ResultSet rs = null;

        if ( isNewDj ) {
            // Make sure no Dj exists with this email
            aSQL = "SELECT COUNT(*) AS ct FROM Dj WHERE email = ?";
            try {
                ps = theConn.prepareStatement(aSQL);
                ps.setString(1, getemail());
                rs = ps.executeQuery();
                if ( rs.next() && rs.getInt(1) > 0 ) {
                    throw new DuplicateException("A DJ with that email already exists.");
                }

            } finally {
                DBUtil.closeRS(rs);
                DBUtil.closeStatement(ps);
            }                

            aSQL
                = "INSERT INTO Dj ( ipaddr, email, password, showname, djname, "
                + "genre, city, state, country, phone, musicdesc, webpage, joindate ) VALUES ( "
                + " ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? ) "
                ;
        } else {
            aSQL
                = "UPDATE Dj SET "
                + "ipaddr = ?, email = ?, password = ?, showname = ?, djname = ?, "
                + "genre = ?, city = ?, state = ?, country = ?, phone = ?, musicdesc = ?, "
                + "webpage = ?, maxweeklyblocks = ?, maxsingleblocks = ? "
                + "WHERE djid = ?";
        }
        
        try {
            int i=1;
            ps = theConn.prepareStatement(aSQL);
            ps.setString(i++,  getipaddr());
            ps.setString(i++,  getemail());
            ps.setString(i++,  getpassword());
            ps.setString(i++,  getshowname());
            ps.setString(i++,  getdjname());
            ps.setInt   (i++,  getgenre());
            ps.setString(i++,  getcity());
            ps.setString(i++,  getstate());
            ps.setString(i++,  getcountry());
            ps.setString(i++,  getphone());
            ps.setString(i++,  getmusicdesc());
            ps.setString(i++,  getwebpage());
            
            if ( !isNewDj ) {
                ps.setInt (i++, getmaxweeklyblocks());
                ps.setInt (i++, getmaxsingleblocks());
                ps.setInt (i++, getdjid());
            } else {
                ps.setLong(i++, System.currentTimeMillis());
           }

            ps.executeUpdate();

            if ( isNewDj ) {
                // Get the inserted id
                ps = theConn.prepareStatement("SELECT currval('dj_djid_seq')");
                rs = ps.executeQuery();
                rs.next();
                setdjid(rs.getInt(1));

            } else if ( isEmailChanged ) {
                InternetAddress aDjInternetEmail
                    = Util.getInternetEmail(getemail(),
                                            getdjname());
                Map aDjMap = new HashMap();
                populateMap(aDjMap);
                aDjMap.put("oldemail", itsOldEmail);
                aDjMap.put("password", itsNewPassword);
                Mailer.queue(Mailer.getFromEmail(),
                             aDjInternetEmail,
                             Mailer.M_EMAIL_CHANGED,
                             aDjMap);
            }

        } finally {
            DBUtil.closeStatement(ps);
        }
    }

    public void updateStats (Connection theConn) throws SQLException {
        String aSQL 
            = "UPDATE Dj SET numstartlate = ?, "
            + "numendearly = ?, numnoshow = ?, totallisteners = ?, "
            + "karma = ?, numbroadcasts = ?, firstbroadcast = ?, numreruns = ? "
            + "WHERE djid = ?"
            ;
        PreparedStatement aPS = null;
        try {
            aPS = theConn.prepareStatement(aSQL);
            int i=1;
            aPS.setInt (i++, getnumstartlate());
            aPS.setInt (i++, getnumendearly());
            aPS.setInt (i++, getnumnoshow());
            aPS.setInt (i++, gettotallisteners());
            aPS.setLong(i++, getkarma());
            aPS.setInt (i++, getnumbroadcasts());
            aPS.setLong(i++, getfirstbroadcast());
            aPS.setInt (i++, getnumreruns());
            aPS.setInt (i++, getdjid());
            aPS.executeUpdate();

        } finally {
            try { aPS.close(); } catch ( Exception e ) {}
        }
    }

    public void lookup (Connection theConn) throws SQLException {

        String aSQL
            = "SELECT Dj.*, Genre.gname "
            + "FROM Dj, Genre "
            + "WHERE Dj.djid = ? "
            + "AND Dj.genre = Genre.gid "
            ;
        PreparedStatement ps = null;
        ResultSet rs = null;

        try {
            ps = theConn.prepareStatement(aSQL);
            ps.setInt(1, getdjid());
            rs = ps.executeQuery();
            if ( rs.next() ) {
                init(rs);
            } else {
                // Should never happen
                throw new SQLException("No DJ exists with id = " + getdjid());
            }

        } finally {
            DBUtil.closeStatement(ps);
            DBUtil.closeRS(rs);
        }
    }

    public String getServerString() {
        String aFirstBroadcast = DBFormat.FormatNormalDate(new Date(getfirstbroadcast()));
        if ( aFirstBroadcast == null || aFirstBroadcast.length() == 0 ) {
            aFirstBroadcast = "None";
        }
        String aJoinDate = DBFormat.FormatNormalDate(new Date(getjoindate()));
        if ( aJoinDate == null || aJoinDate.length() == 0 ) {
            aJoinDate = "None";
        }
        String aWebPage = getwebpage();
        if ( aWebPage == null || aWebPage.length() == 0 ) {
            aWebPage = "None";
        }
        return
            "|" + getdjid() +
            "|" + getpassword() + 
            "|" + getkarma() + 
            "|" + getgenre() + 
            "|" + getipaddr() + 
            "|" + getemail() + 
            "|" + getshowname() + 
            "|" + getdjname() + 
            "|" + getgenreval() + 
            "|" + aJoinDate + 
            "|" + aFirstBroadcast + 
            "|" + getnumbroadcasts() + 
            "|" + getnumreruns() + 
            "|" + gettotallisteners() + 
            "|" + getnumsubs() + 
            "|" + getnumstartlate() + 
            "|" + getnumendearly() + 
            "|" + getnumnoshow() + 
            "|" + getcity() +  
            "|" + getstate() + 
            "|" + getcountry() + 
            "|" + aWebPage + 
            ""
            ;
    }

    public int login ( Connection theConn, 
                       String theEmail, 
                       String thePassword ) throws SQLException {

        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL
            = "SELECT Dj.*, Genre.gname "
            + "FROM Dj, Genre "
            + "WHERE email = ? "
            + "AND Dj.genre = Genre.gid "
            ;
        try {
            aPS = theConn.prepareStatement(aSQL);
            aPS.setString(1, theEmail);
            aRS = aPS.executeQuery();
            if ( !aRS.next() ) {
                return LOGIN_NOSUCHUSER;
            } else {
                String aPassword = aRS.getString("password");
                debug.out("DjBean.login: comparing DB='" + aPassword + "' to web='" + thePassword + "'");
                if ( aPassword != null && aPassword.equals(thePassword) ) {
                    init(aRS);
                    return LOGIN_OK;
                } else {
                    return LOGIN_BADPASSWORD;
                }
            }
        } finally { 
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
        }
    }
}
