/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/beans/onair/OnAirBean.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: OnAirBean.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.beans.onair;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.sql.SQLException;
import java.util.Map;
import org.opendj.util.Util;
import org.opendj.util.DBFormat;
import org.opendj.util.DBUtil;
import org.opendj.util.Config;
import org.opendj.util.StringUtil;

public class OnAirBean {

    private int       psid         = -1;
    private int       channelid    = -1;
    private String    channelpath  = null;
    private String    channelname  = null;
    private int       djid         = -1;
    private int       edjid         = -1;
    private String    ipaddr       = null;
    private String    djemail       = null;
    private String    djname       = null;
    private String    showname     = null;
    private int       genre        = -1;
    private String    genreval     = null;
    private int       archid        = -1;
    private long      starttime    = -1;
    private long      endtime      = -1;
    private int       status       = -1;
    private int       numweeks     = -1;

    // Extended fields
    private String musicdesc = null;
    private int    numlisteners = -1;

    private boolean isOnAirNow = false;
    private boolean isOnAirLater = false;

    // Fields only filled out for reruns
    private long   origstarttime = -1;

    private static int RERUN_DJID = -1;
    private static String LIVE_BROADCAST_STRING = Util.EMPTY_STRING;
    
    static {
        RERUN_DJID = Integer.parseInt(Config.global().getProperty(Config.PROP_RERUNDJID));
        LIVE_BROADCAST_STRING = Config.global().getProperty(Config.PROP_LIVEBROADCASTSTRING);
        if ( LIVE_BROADCAST_STRING == null ) LIVE_BROADCAST_STRING = Util.EMPTY_STRING;
    }

    public OnAirBean() {
        isOnAirNow = false;
        isOnAirLater = false;
    }

    public int getpsid() { return psid; }
    public void setpsid(int psid) { this.psid = psid; }

    public int getchannelid() { return channelid; }
    public void setchannelid(int channelid) { this.channelid = channelid; }

    public String getchannelpath() { return channelpath; }
    public void setchannelpath(String channelpath) { this.channelpath = channelpath; }

    public String getchannelname() { return channelname; }
    public void setchannelname(String channelname) { this.channelname = channelname; }

    public int getdjid() { 
        if ( djid == RERUN_DJID ) {
            return edjid;
        }
        return djid;
    }
    public int getrealdjid() {
        return djid;
    }
    public void setdjid(int djid) { this.djid = djid; }

    public boolean getisrerun() {
        return ( djid == RERUN_DJID );
    }
    public String getisrerunstring() {
        if ( djid == RERUN_DJID ) {
            return "<br><i>Originally Broadcast on " 
                + DBFormat.FormatRelativeTimestamp( new Timestamp(origstarttime) )
                + "</i>";
        }
        return Util.EMPTY_STRING;
    }
    public String getisliveshowstring() {
        if ( djid == RERUN_DJID ) {
            return Util.EMPTY_STRING;
        }
        return LIVE_BROADCAST_STRING;
    }

    public int getedjid() { return edjid; }
    public void setedjid(int edjid) { this.edjid = edjid; }

    public String getipaddr() { return ipaddr; }
    public void setipaddr(String ipaddr) { this.ipaddr = ipaddr; }

    public String getdjemail() { return djemail; }
    public void setdjemail(String djemail) { this.djemail = djemail; }

    public String getdjname() { return djname; }
    public void setdjname(String djname) { this.djname = djname; }

    public String getshowname() { return showname; }
    public void setshowname(String showname) { this.showname = showname; }

    public int getgenre() { return genre; }
    public void setgenre(int genre) { this.genre = genre; }

    public String getgenreval() { return genreval; }
    public void setgenreval(String genreval) { this.genreval = genreval; }

    public int getarchid() { return archid; }
    public void setarchid(int archid) { this.archid = archid; }

    public int  getstatus() { return status; }
    public void setstatus(int status) { this.status = status; }

    public long getstarttime() { return starttime; }
    public void setstarttime(long starttime) { this.starttime = starttime; }

    public long getendtime() { return endtime; }
    public void setendtime(long endtime) { this.endtime = endtime; }

    public int getnumweeks() { return numweeks; }
    public void setnumweeks(int numweeks) { this.numweeks = numweeks; }

    public long getorigstarttime() { return origstarttime; }
    public void setorigstarttime(long origstarttime) { this.origstarttime = origstarttime; }

    public String getstartstring() {
        return DBFormat.FormatRelativeTimestamp( new Timestamp(starttime) );
    }
    public String getendstring() {
        return DBFormat.FormatRelativeTimestamp( new Timestamp(endtime) );
    }

    public String getstartstringex() {
        return DBFormat.FormatRelativeTimestampEx( new Timestamp(starttime) );
    }
    public String getendstringex() {
        return DBFormat.FormatRelativeTimestampEx( new Timestamp(endtime) );
    }

    public long getstarttimelong() { return starttime; }
    public void setstarttimelong( long starttime ) { this.starttime = starttime; }
    public long getendtimelong() { return endtime; }
    public void setendtimelong( long endtime ) { this.endtime = endtime; }

    public long getduration () { return (endtime - starttime); }

    public String getmusicdesc() { return musicdesc; }
    public void setmusicdesc(String musicdesc) { this.musicdesc = musicdesc; }

    public int getnumlisteners() { return numlisteners; }
    public void setnumlisteners(int numlisteners) { this.numlisteners = numlisteners; }

    public boolean getIsOnAirNow() { return isOnAirNow; }
    public boolean getIsOnAirLater() { return isOnAirLater; }
    public boolean getAlreadyHasSub() { return djid != edjid; }

    public void init ( ResultSet theRS ) throws SQLException {
        setpsid         (theRS.getInt   ("psid"));
        setchannelid    (theRS.getInt   ("channelid"));
        setchannelname  (theRS.getString("channelname"));
        setchannelpath  (theRS.getString("channelpath"));
        setdjid         (theRS.getInt   ("djid"));
        setedjid        (theRS.getInt   ("edjid"));
        setipaddr       (theRS.getString("ipaddr"));
        setdjemail      (theRS.getString("djemail"));
        setdjname       (theRS.getString("djname"));
        setshowname     (theRS.getString("showname"));
        setgenre        (theRS.getInt   ("genre"));
        setgenreval     (theRS.getString("genreval"));
        setarchid       (theRS.getInt   ("archid"));
        setstarttime    (theRS.getLong  ("starttime"));
        setendtime      (theRS.getLong  ("endtime"));
        setstatus       (theRS.getInt   ("status"));
        setnumweeks     (theRS.getInt   ("numweeks"));

        long aNowTime = System.currentTimeMillis();
        isOnAirNow = ( aNowTime >= starttime && aNowTime <= endtime );
        isOnAirLater = ( isOnAirNow == false && aNowTime <= starttime );
    }

    /** @deprecated NOTE: THE QUERY THAT WAS INTENDED TO BE USED HERE SOMETIMES
        CRAPS OUT MYSQL.  DON'T USE THIS METHOD **/
    public void initExtended( ResultSet theRS ) throws SQLException {
        init(theRS);
        setmusicdesc(theRS.getString("musicdesc"));
    }

    public void populateMap ( Map m ) {
        m.put("psid", new Integer(getpsid()));
        m.put("channelid", new Integer(getchannelid()));
        m.put("channelname", getchannelname());
        m.put("channelpath", getchannelpath());
        m.put("djid", new Integer(getdjid()));
        m.put("edjid", new Integer(getedjid()));
        m.put("ipaddr", getipaddr());
        m.put("djemail", getdjemail());
        m.put("djname", getdjname());
        m.put("showname", getshowname());
        m.put("genre", new Integer(getgenre()));
        m.put("genreval", getgenreval());
        m.put("archid", new Integer(getarchid()));
        m.put("starttime", new Long(getstarttime()));
        m.put("endtime", new Long(getendtime()));
        m.put("starttimestring", getstartstring());
        m.put("endtimestring", getendstring());
        m.put("numweeks", new Integer(getnumweeks()));
    }

    public void lookup ( Connection theConn ) throws SQLException {

        String aSQL
            = "SELECT * "
            + "FROM PublishedSchedule "
            + "WHERE psid = ? "
            ;
        PreparedStatement ps = null;
        ResultSet rs = null;

        try {
            ps = theConn.prepareStatement(aSQL);
            ps.setInt(1, getpsid());
            rs = ps.executeQuery();
            if ( rs.next() ) {
                init(rs);
            }

        } finally {
            DBUtil.closeStatement(ps);
            DBUtil.closeRS(rs);
        }
    }

    public boolean isPredecessorTo ( OnAirBean theOnAirBean ) {
        return ( getchannelid() == theOnAirBean.getchannelid() &&
                 getendtime() == theOnAirBean.getstarttime() );
    }

    public boolean isSuccessorTo ( OnAirBean theOnAirBean ) {
        return ( getchannelid() == theOnAirBean.getchannelid() &&
                 getstarttime() == theOnAirBean.getendtime() );
    }

    public void loadRerunData ( Connection theConn ) throws SQLException {

        PreparedStatement ps = null;
        ResultSet rs = null;
        String aSQL
            = "SELECT genre, genreval, starttime, djcomments "
            + "FROM ArchivedShow "
            + "WHERE archid = ? "
            ;
        String aMusicDesc = null;
        try {
            ps = theConn.prepareStatement(aSQL);
            ps.setInt(1, archid);
            rs = ps.executeQuery();
            if ( rs.next() ) {
                genre = rs.getInt(1);
                genreval = rs.getString(2);
                origstarttime = rs.getLong(3);
                aMusicDesc = rs.getString(4);
                if ( StringUtil.nz(aMusicDesc).length() > 0 ) {
                    musicdesc = aMusicDesc;
                }
            } else {
                // XXX Log this?
            }
        } finally {
            DBUtil.closeRS(rs);
            DBUtil.closeStatement(ps);
        }
    }

    public String toString() {
        return "[OnAirBean: DJ='" + getdjname() 
            + "', start=" + getstartstring() 
            + ", end=" + getendstring() + "]";
    }
}
