/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/beans/onair/OnAirStatusBean.java,v 1.1.1.1 2001/09/29 20:38:49 jonathan Exp $
 * $Log: OnAirStatusBean.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.beans.onair;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.opendj.util.Util;
import org.opendj.util.DBUtil;
import org.opendj.util.SchedBlock;

public class OnAirStatusBean {

    private int    itsPsid           = -1;
    private String itsChannelpath    = null;
    private int    itsDjid           = -1;
    private int    itsBitrate        = -1;
    private int    itsEdjid          = -1;
    private long   itsStarttime      = -1;
    private long   itsEndtime        = -1;
    private long   itsOnairtime      = -1;
    private long   itsLastonairtime  = -1;
    private long   itsLastsampletime = -1;
    private int    itsSamplecount    = -1;
    private int    itsOnaircount     = -1;
    private int    itsListenercount  = -1;
    private int    itsRecentlcount   = -1;
    private int    itsNumweeks       = -1;
    private int    itsStatus         = -1;

    public OnAirStatusBean() {}

    public int    getPsid() { return itsPsid; }
    public void   setPsid(int thePsid) { itsPsid = thePsid; }

    public String getChannelpath() { return itsChannelpath; }
    public void   setChannelpath(String theChannelpath) { itsChannelpath = theChannelpath; }

    public int    getDjid() { return itsDjid; }
    public void   setDjid(int theDjid) { itsDjid = theDjid; }

    public int    getBitrate() { return itsBitrate; }
    public void   setBitrate(int theBitrate) { itsBitrate = theBitrate; }

    public int    getEdjid() { return itsEdjid; }
    public void   setEdjid(int theEdjid) { itsEdjid = theEdjid; }

    public long   getStarttime() { return itsStarttime; }
    public void   setStarttime(long theStarttime) { itsStarttime = theStarttime; }

    public long   getEndtime() { return itsEndtime; }
    public void   setEndtime(long theEndtime) { itsEndtime = theEndtime; }

    public long   getOnairtime() { return itsOnairtime; }
    public void   setOnairtime(long theOnairtime) { itsOnairtime = theOnairtime; }

    public long   getLastonairtime() { return itsLastonairtime; }
    public void   setLastonairtime(long theLastonairtime) { itsLastonairtime = theLastonairtime; }

    public long   getLastsampletime() { return itsLastsampletime; }
    public void   setLastsampletime(long theLastsampletime) { itsLastsampletime = theLastsampletime; }

    public int    getSamplecount() { return itsSamplecount; }
    public void   setSamplecount(int theSamplecount) { itsSamplecount = theSamplecount; }

    public int    getOnaircount() { return itsOnaircount; }
    public void   setOnaircount(int theOnaircount) { itsOnaircount = theOnaircount; }

    public int    getListenercount() { return itsListenercount; }
    public void   setListenercount(int theListenercount) { itsListenercount = theListenercount; }

    public int    getRecentlcount() { return itsRecentlcount; }
    public void   setRecentlcount(int theRecentlcount) { itsRecentlcount = theRecentlcount; }

    public int    getNumweeks() { return itsNumweeks; }
    public void   setNumweeks(int theNumweeks) { itsNumweeks = theNumweeks; }

    public int    getStatus() { return itsStatus; }
    public void   setStatus(int theStatus) { itsStatus = theStatus; }

    public float getAverageListeners () {
        if ( itsListenercount <= 0 ) return 0;
        return ((float) itsListenercount) / ((float) itsSamplecount);
    }
    public float getAverageDisconnect (long theSampleMillis) {
        if ( itsSamplecount <= 0 ) return 0;
        // Figure out how many samples took place while they were late or left early
        long aLateSamples = getLateMillis() / theSampleMillis;
        long anEarlySamples = getEarlyMillis() / theSampleMillis;
        long aMainSampleCount = itsSamplecount - aLateSamples - anEarlySamples;
        return (((float) aMainSampleCount) - ((float) itsOnaircount)) / ((float) aMainSampleCount);
    }
    public int getNumDisconnects () {
        return itsSamplecount - itsOnaircount;
    }
    public long getLateMillis() {
        return itsOnairtime - itsStarttime;
    }
    public long getEarlyMillis() {
        return itsEndtime - itsLastonairtime;
    }
    public long getNumBlocks() {
        return (itsEndtime - itsStarttime)/Util.MILLIS_IN_MINUTE/SchedBlock.SCHEDBLOCK_MINUTES;
    }
    public boolean getWasSubstitute() {
        return ( itsDjid != itsEdjid );
    }
    
    public void init ( ResultSet theRS ) throws SQLException {
        setChannelpath   (theRS.getString("channelpath"));
        setPsid          (theRS.getInt   ("psid"));
        setDjid          (theRS.getInt   ("djid"));
        setBitrate       (theRS.getInt   ("bitrate"));
        setEdjid         (theRS.getInt   ("edjid"));
        setStarttime     (theRS.getLong  ("starttime"));
        setEndtime       (theRS.getLong  ("endtime"));
        setOnairtime     (theRS.getLong  ("onairtime"));
        setLastonairtime (theRS.getLong  ("lastonairtime"));
        setLastsampletime(theRS.getLong  ("lastsampletime"));
        setSamplecount   (theRS.getInt   ("samplecount"));
        setOnaircount    (theRS.getInt   ("onaircount"));
        setListenercount (theRS.getInt   ("listenercount"));
        setRecentlcount  (theRS.getInt   ("recentlcount"));
        setNumweeks      (theRS.getInt   ("numweeks"));
        setStatus        (theRS.getInt   ("status"));
    }

    public boolean lookup ( Connection theConn ) throws SQLException {

        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL = "SELECT * FROM OnAirStatus WHERE channelpath = ?";

        try {
            aPS = theConn.prepareStatement(aSQL);
            aPS.setString(1, itsChannelpath);
            aRS = aPS.executeQuery();
            if ( aRS.next() ) {
                init(aRS);
                return true;
            }
            return false;

        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
        }
    }

    public String toString() {
        return "[OnAirStatusBean: psid=" + itsPsid
            + ", channelpath='" + itsChannelpath + "'"
            + ", djid=" + itsDjid
            + ", edjid=" + itsEdjid + "]";
    }
}
