/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/beans/schedblock/SchedBlockBean.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: SchedBlockBean.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.beans.schedblock;

import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.opendj.util.Util;
import org.opendj.util.DBFormat;
import org.opendj.util.SchedBlock;

public class SchedBlockBean {

    private int    itsBID           = -1;
    private int    itsChannelid     = -1;
    private String itsChannelidval  = null;
    private int    itsDay           = -1;
    private String itsDayval        = null;
    private int    itsAttributes    = -1;
    private int    itsStarttime     = -1;

    public SchedBlockBean () {}

    public int    getbid() { return itsBID; }
    public void   setbid(int theBID) { itsBID = theBID; }

    public int    getchannelid() { return itsChannelid; }
    public void   setchannelid(int theChannelid) { itsChannelid = theChannelid; }

    public String getchannelidval() { return itsChannelidval; }
    public void   setchannelidval(String theChannelidval) { itsChannelidval = theChannelidval; }

    public int    getday() { return itsDay; }
    public void   setday(int theDay) { itsDay = theDay; }

    public String getdayval() { return itsDayval; }
    public void   setdayval(String theDayval) { itsDayval = theDayval; }

    public int    getstarttime() { return itsStarttime; }
    public void   setstarttime(int theStarttime) { itsStarttime = theStarttime; }

    public String getstarttimeval() {
        return DBFormat.FormatStartTime(itsStarttime);
    }

    public int    getattributes() { return itsAttributes; }
    public void   setattributes(int theAttributes) { itsAttributes = theAttributes; }

    public void init ( ResultSet rs ) throws SQLException {
        setbid           (rs.getInt   ("bid"));
        setchannelid     (rs.getInt   ("channelid"));
        setchannelidval  (rs.getString("path"));
        setattributes    (rs.getInt   ("attributes"));
        setday           (rs.getInt   ("day"));
        setdayval        (Util.DAY_NAMES[getday()]);
        setstarttime     (rs.getInt   ("starttime"));
    }

    public void initBasic ( ResultSet rs ) throws SQLException {
        setbid           (rs.getInt   ("bid"));
        setchannelid     (rs.getInt   ("channelid"));
        setattributes    (rs.getInt   ("attributes"));
        setday           (rs.getInt   ("day"));
        setdayval        (Util.DAY_NAMES[getday()]);
        setstarttime     (rs.getInt   ("starttime"));
    }
    
    public String toString() {
        return "[SchedBlockBean bid=" + getbid() 
            + ",channelid=" + getchannelid() 
            + ",day=" + getday() 
            + ",starttime=" + getstarttime() + "]";
    }

    public boolean getIsModerated() {
        return (getattributes() & SchedBlock.ATTR_UNMODERATED) == 0;
    }

    public boolean getIsWeekly() {
        return (getattributes() & SchedBlock.ATTR_ONETIME)     == 0;
    }

    public boolean getIsSubdj() {
        return (getattributes() & SchedBlock.ATTR_SUBDJ)       != 0;
    }

    public boolean getIsConvertible() {
        return (getattributes() & SchedBlock.ATTR_NONCONVERTIBLE) == 0;
    }

}
