/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/publish/PublishSchedule.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: PublishSchedule.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.publish;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.List;
import java.util.Vector;
import org.opendj.util.Util;
import org.opendj.util.DBUtil;

public class PublishSchedule {

    public static void main ( String[] args ) {
        PublishSchedule ps = new PublishSchedule();
        try {
            ps.publish();
        } catch ( Exception e ) {
            log(e);
        }
    }

    public PublishSchedule() {}

    public void publish() {

        Connection aConn = null;
        PreparedStatement aSelectPS = null;
        ResultSet aRS = null;
        String aSelectSQL
            = "SELECT Scheduled.sid, Scheduled.djid, Dj.email AS djemail, Dj.djname, "
            + "Dj.showname, Dj.genre, Dj.ipaddr, Genre.gname AS genreval, "
            + "Scheduled.issingle, Scheduled.singledate, "
            + "ScheduleBlock.channelid, ScheduleBlock.day, ScheduleBlock.starttime, "
            + "Scheduled.startlease, Scheduled.endlease, "
            + "Channel.path AS channelpath, Channel.name AS channelname "
            + "FROM Scheduled, ScheduleBlock, Dj, Genre, Channel "
            + "WHERE Scheduled.bid = ScheduleBlock.bid "
            + "AND Scheduled.djid = Dj.djid "
            + "AND Dj.genre = Genre.gid "
            + "AND ScheduleBlock.channelid = Channel.cid "
            + "AND Scheduled.ispublished = 0 "
            + "AND Scheduled.isactive = 1 "
            + "ORDER BY ScheduleBlock.channelid, Scheduled.singledate, "
            + "ScheduleBlock.day, ScheduleBlock.starttime "
            ;

        List aSIDList = new Vector();
        PublishSlot aFirstSlotInShow = null;
        PublishSlot aCurrentSlot = null;
        PublishSlot aLastSlot    = null;
        try {
            aConn = DBUtil.getConnection();
            aSelectPS = aConn.prepareStatement(aSelectSQL);
            aRS = aSelectPS.executeQuery();

            while (aRS.next()) {
                aCurrentSlot = new PublishSlot();
                aCurrentSlot.init(aRS);
                
                // Collect all Scheduled entities that are part of the same
                // show
                if ( aCurrentSlot.isNextPartOfSameShow(aLastSlot) ) {
                    // Do nothing
                    
                } else if ( aLastSlot != null ) {
                    // aCurrentSlot is a different show, so publish all blocks
                    // in the sidlist
                    aFirstSlotInShow.publish(aSIDList, aConn);
                    aSIDList.clear();
                    aFirstSlotInShow = aCurrentSlot;
                    
                } else {
                    aFirstSlotInShow = aCurrentSlot;
                }
                aSIDList.add(new Integer(aCurrentSlot.sid));
                aLastSlot = aCurrentSlot;
            }

            // Publish the last show, if there was one
            if ( aSIDList.size() > 0 ) {
                aFirstSlotInShow.publish(aSIDList, aConn);
            }

        } catch ( Exception e ) {
            log(e);

        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aSelectPS);
            DBUtil.closeConnection(aConn);
        }
    }

    private static void log ( Exception e ) {
        log(Util.getStackTrace(e));
    }

    private static void log ( String s ) {
        System.err.println("[PublishSchedule] " + s);
    }

}
