/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/ChannelTime.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: ChannelTime.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;

public class ChannelTime extends org.opendj.share.ChannelTime {

    public ChannelTime() {
        super();
    }

    public ChannelTime( String name, String path, long delta, long length ) {
        super(name, path, delta, length);
    }

    public ChannelTime( String name, String path, long delta, long length, int special ) {
        super(name, path, delta, length, special);
    }

    public String getServerString() {
        return "|" + name + "|" + path + "|" + delta + "|" + length + "|" + special;
    }

    public static ChannelTime init( ResultSet theRS ) throws SQLException {

        // Initialize the data
        String aName = theRS.getString("channelname");
        String aPath = theRS.getString("channelpath");
        long aStart  = theRS.getLong("starttime");
        long anEnd   = theRS.getLong("endtime");

        // Create derived attributes
        long aNowTime = System.currentTimeMillis();
        long aDelta = aStart - aNowTime;
        long aLength = anEnd - aStart;
        int aSpecial = ChannelTime.NONE;

        // If the start is before now, and the end is after now, mark
        // this a ONAIR
        if ( aDelta < 0 && anEnd > aNowTime ) {
            aSpecial = ChannelTime.ONAIR;
        }
        return new ChannelTime(aName, aPath, aDelta, aLength, aSpecial);
    }
}
