/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/Config.java,v 1.1.1.1 2001/09/29 20:38:48 jonathan Exp $
 * $Log: Config.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;
import javax.servlet.*;
import javax.servlet.http.*;

import org.opendj.util.log.Logger;

public class Config extends HttpServlet {

    public static final String PROP_SMTPHOST             = "opendj.mail.smtp.host";
    public static final String PROP_MAILTEMPLATEDIR      = "opendj.mail.template.dir";
    public static final String PROP_FROMEMAIL            = "opendj.mail.email.generic.from.addr";
    public static final String PROP_FROMNAME             = "opendj.mail.email.generic.from.name";
    public static final String PROP_ADMINFROMEMAIL       = "opendj.mail.email.admin.from.addr";
    public static final String PROP_ADMINFROMNAME        = "opendj.mail.email.admin.from.name";
    public static final String PROP_ADMINTOEMAIL         = "opendj.mail.email.admin.to.addr";
    public static final String PROP_ADMINTONAME          = "opendj.mail.email.admin.to.name";
    public static final String PROP_WEBMTOEMAIL          = "opendj.mail.email.webmaster.to.addr";
    public static final String PROP_WEBMTONAME           = "opendj.mail.email.webmaster.to.name";
    public static final String PROP_LOGGERFROMEMAIL      = "opendj.mail.email.logger.from.addr";
    public static final String PROP_LOGGERTOEMAIL        = "opendj.mail.email.logger.to.addr";

    public static final String PROP_TEMP_DIR             = "opendj.tmp.dir";

    public static final String PROP_DB_DRIVER            = "opendj.jdbc.driver";
    public static final String PROP_DB_URL               = "opendj.jdbc.url";
    public static final String PROP_DB_USER              = "opendj.jdbc.user";
    public static final String PROP_DB_PASSWORD          = "opendj.jdbc.password";
    
    public static final String PROP_CHATSERVER_PORT      = "chatserver.port";
    public static final String PROP_DJSERVER_IP          = "djserver.ip";
    public static final String PROP_DJSERVER_PORT        = "djserver.port";
    public static final String PROP_ICECAST_IP           = "djserver.icecast.ip";
    public static final String PROP_ICECAST_PORT         = "djserver.icecast.encoder.port";
    public static final String PROP_ICECAST_PASSWORD     = "djserver.icecast.encoder.password";
    public static final String PROP_MAX_BITRATE          = "djserver.icecast.bitrate.max";

    public static final String PROP_LOGDIR               = "opendj.log.dir";
    public static final String PROP_ARCHIVEDIR           = "opendj.archive.dir";
    public static final String PROP_PLSDIR               = "opendj.archive.pls.dir";
    public static final String PROP_PLSLINK              = "opendj.archive.pls.link";
    public static final String PROP_PLSLIVELINKPREFIX    = "opendj.live.pls.link.prefix";
    public static final String PROP_PLSLIVEDIR           = "opendj.live.pls.dir";
    public static final String PROP_INTERNAL_NET_MATCH   = "opendj.ipmatch.internal";
    public static final String PROP_INTERNAL_GW_MATCH    = "opendj.ipmatch.gateway";
    public static final String PROP_SERVICECLASSES       = "opendj.services.classes";
    public static final String PROP_SERVICEPORT          = "opendj.services.listener.port";
    public static final String PROP_SERVICEAUTHPASSWORD  = "opendj.services.auth.password";

    public static final String PROP_LIVEBROADCASTSTRING  = "opendj.livebroadcast.string";

    public static final String PROP_MAXAVAILRERUNS       = "opendj.rerun.scheduler.maxavailreruns";
    public static final String PROP_LASTAIRLIMITHOURS    = "opendj.rerun.scheduler.lastairlimithours";
    public static final String PROP_MINRERUNKARMA        = "opendj.rerun.scheduler.minrerunkarma";
    public static final String PROP_RERUNIP              = "opendj.rerun.ip";
    public static final String PROP_RERUNDJID            = "opendj.rerun.djid";
    public static final String PROP_RERUNDJEMAIL         = "opendj.rerun.email";
    public static final String PROP_RERUNDJPASSWORD      = "opendj.rerun.password";
    public static final String PROP_LIMITCONNECTTOSERVER = "opendj.rerun.player.limit.connection";
    public static final String PROP_LIMITSTREAMSETUP     = "opendj.rerun.player.limit.streamsetup";
    public static final String PROP_RERUNASKINTERVAL     = "opendj.rerun.player.askinterval";
    public static final String PROP_RERUNDEADAIRMP3DIR   = "opendj.rerun.player.deadairmp3dir";

    public static final String PROP_REQSUBLIMITHOURS     = "opendj.schedule.sub.request.hours";
    public static final String PROP_MINSUBACCEPTHOURS    = "opendj.schedule.sub.accept.hours";
    public static final String PROP_RERUNSCHEDLIMITHOURS = "opendj.schedule.rerun.hours";
    public static final String PROP_RERUNPCTLIVELIMIT    = "opendj.schedule.rerun.pct.live.limit";

    public static final String PROP_ONBATTERY_COMMAND    = "opendj.power.onbattery.cmd";
    public static final String PROP_BATTCHARGE_COMMAND   = "opendj.power.battcharge.cmd";
    public static final String PROP_TIMELEFT_COMMAND     = "opendj.power.timeleft.cmd";
    public static final String PROP_POWERMONITOR_MINS    = "opendj.power.monitor.mins";
    public static final String PROP_FILENORMAL_COMMAND   = "opendj.power.normal.cmd";
    public static final String PROP_FILELOSS_COMMAND     = "opendj.power.loss.cmd";

    // This is the ALL-IMPORTANT file location that is the ONLY thing
    // that should be hard-coded in the whole system!
    public static final String MAIN_PROPFILE = "/etc/opendj/site.properties";

    // This is an initparam that should be defined in the web.xml for the webapp
    public static final String SERVLET_INITPARAM_LOGDIR = "logDir";

    public static final String LOGFILE_NAME = "opendj_tomcat";

    private Properties itsProps = null;
    private boolean isInit = false;

    private static Config itsGlobal = null;

    public String getProperty ( String thePropName ) {
        return itsProps.getProperty(thePropName);
    }

    public void loadProperties () 
        throws IOException {

        if ( !isInit ) {
            synchronized (itsProps) { 
                if ( !isInit ) { 
                    String aFileName = null;
                    FileInputStream fi = null;
                    try {
                        // debug.out("creating fi (" + MAIN_PROPFILE + ")...");
                        fi = new FileInputStream(MAIN_PROPFILE);
                        // debug.out("loading props from fi...");
                        itsProps.load(fi);
                        // debug.out("props loaded OK");
                    } finally {
                        try { fi.close(); } catch ( Exception e ) {}
                    }
                    isInit = true;
                }
            }
        }
    }

    public void init ( ServletConfig theConfig ) {
        debug.out("Config.init called...");
        synchronized (itsProps) {
            if ( !isInit ) {
                try {
                    loadProperties();
                } catch ( Exception e ) {
                    debug.out("Shit!" + e.toString());
                    e.printStackTrace();
                    if ( theConfig != null ) {
                        theConfig.getServletContext().log("ERROR INITIALIZING Config: " +
                                                          e.toString() + "\n" +
                                                          Util.getStackTrace(e));
                    }
                }

                // If theConfig is not null, then we're
                // being invoked as a startup servlet, so
                // check to see if the "logDir" initparam was set.
                if ( theConfig != null ) {
                    String aLogDir = theConfig.getInitParameter(SERVLET_INITPARAM_LOGDIR);
                    if ( aLogDir != null ) {
                        Logger.initGlobal( aLogDir, LOGFILE_NAME );
                    }

                    // Also, override the email template dir to
                    // use the templates deployed with the webapp
                    itsProps.setProperty(PROP_MAILTEMPLATEDIR, 
                                         theConfig.getServletContext().getRealPath("WEB-INF/resources/emailtemplates/"));
                }

            }
        }
    }

    public void destroy () {}

    public static Config global() { 
        if ( itsGlobal == null ) {
            itsGlobal = new Config();
            itsGlobal.init(null);
        }
        return itsGlobal; 
    }

    public Config() {
        itsProps = new Properties();
        isInit = false;
        if ( itsGlobal == null ) {
            itsGlobal = this;
        }
    }

    public Properties getProperties () {
        return new Properties(itsProps);
    }
}
