/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.3 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/DBFormat.java,v 1.3 2001/10/21 20:53:53 jonathan Exp $
 * $Log: DBFormat.java,v $
 * Revision 1.3  2001/10/21 20:53:53  jonathan
 * Removed GPL notices from files that are not GPLd.
 *
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.sql.*;
import java.util.Date;
import java.util.Map;
import java.text.SimpleDateFormat;
import java.text.DecimalFormat;
import java.text.DateFormat;

public class DBFormat {

    private static final String[] STATES
        = { "AK", "AL", "AR", "AZ", "CA", "CO", "CT", "DC", "DE", 
            "FL", "GA", "GU", "HI", "IA", "ID", "IL", "IN", "KS", 
            "KY", "LA", "MA", "MI", "MN", "MO", "MS", "MT", "NC",
            "ND", "NE", "NH", "NJ", "NM", "NV", "NY", "OH", "OK", 
            "OR", "PA", "PR", "RI", "SC", "SD", "TN", "TX", "UT",
            "VA", "VI", "VT", "WA", "WI", "WV", "WY" };

    private static final SimpleDateFormat 
        itsSQLDateFormat = new SimpleDateFormat("yyyy-MM-dd");

    private static final SimpleDateFormat 
        itsSQLDateTimeFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private static SimpleDateFormat itsUIDateFormat = null;
    private static SimpleDateFormat itsUIExtDateFormat = null;
    private static SimpleDateFormat itsUIExtDateTimeFormat = null;
    private static SimpleDateFormat itsUIExtYearDateTimeFormat = null;
    private static SimpleDateFormat itsUIDateTimeFormat = null;
    private static SimpleDateFormat itsUITimeFormat = null;

    private static DecimalFormat 
        itsDecimalFormat = new DecimalFormat("##########.##");

    private DBFormat() {}

    static {
        String aUIDateFormat = null;
        String aUIDateTimeFormat = null;
        String aUITimeFormat = null;
        String aUIExtDateFormat = null;
        String aUIExtDateTimeFormat = null;
        String aUIExtYearDateTimeFormat = null;

        aUIDateFormat = "M/d/yyyy";
        itsUIDateFormat = new SimpleDateFormat(aUIDateFormat);

        aUIDateTimeFormat = "M/d/yyyy HH:mm:ss";
        itsUIDateTimeFormat = new SimpleDateFormat(aUIDateTimeFormat);

        aUITimeFormat = "HH:mm";
        itsUITimeFormat = new SimpleDateFormat(aUITimeFormat);

        aUIExtDateFormat = "MMMM d";
        itsUIExtDateFormat = new SimpleDateFormat(aUIExtDateFormat);

        aUIExtDateTimeFormat = "MMMM d, HH:mm";
        itsUIExtDateTimeFormat = new SimpleDateFormat(aUIExtDateTimeFormat);

        aUIExtYearDateTimeFormat = "MMMM d, yyyy HH:mm";
        itsUIExtYearDateTimeFormat = new SimpleDateFormat(aUIExtYearDateTimeFormat);
    }

    public static String FormatSQLString( String theValue ) {
        if ( theValue == null ) {
            return "NULL";
        }
        return "'" + DBFormat.replaceCharInString( theValue, '\'', "\\'" ) + "'";
    }

    public static String FormatSQLLong( Long theValue ) {
        if ( theValue == null ) {
            return "NULL";
        }
        return theValue.toString();
    }

    public static String FormatSQLDouble( Double theValue ) {
        if ( theValue == null ) {
            return "NULL";
        }
        return itsDecimalFormat.format(theValue.doubleValue());
    }

    public static String FormatUIDouble( String theValue ) {
        if ( theValue == null ) {
            return "NULL";
        }
        try {
            return itsDecimalFormat.format(new Double(theValue).doubleValue());
        } catch ( Exception e ) {
            return Util.EMPTY_STRING;
        }
    }

    public static String FormatSQLDate( java.util.Date theDate ) {
        if ( theDate == null ) { 
            return "NULL";
        }
        return "'" + itsSQLDateFormat.format(theDate) + "'";
    }

    public static String FormatSQLDateTime( java.util.Date theDate ) {
        if ( theDate == null ) { 
            return "NULL";
        }
        return "'" + itsSQLDateTimeFormat.format(theDate) + "'";
    }

    public static String FormatNormalDate( java.util.Date theDate ) {
        
        // com.aeon23.compsplus.cpdebug.out("formatting normal date " + theDate.getTime());

        if ( theDate == null ) { 
            return Util.EMPTY_STRING;
        }
        // com.aeon23.compsplus.cpdebug.out("formatting normal date " + theDate + " (" + theDate.getTime() + ") = " + itsUIDateFormat.format(theDate));

        if ( theDate.getTime() <= 0 ) {
            // com.aeon23.compsplus.cpdebug.out("returinng empty...");
            return Util.EMPTY_STRING;
        }
        return itsUIDateFormat.format(theDate);
    }
    public static String FormatNormalDate( String theDate ) {
        
        // com.aeon23.compsplus.cpdebug.out("formatting normal date " + theDate);

        if ( theDate == null || theDate.equals(Util.EMPTY_STRING) ) { 
            return Util.EMPTY_STRING;
        }
        try {
            return itsUIDateFormat.format(itsSQLDateFormat.parse(theDate));
        } catch (Exception e) {
            return Util.EMPTY_STRING;
        }
    }


    public static String FormatExtDate( java.util.Date theDate ) {
        
        if ( theDate == null ) { 
            return Util.EMPTY_STRING;
        }
        if ( theDate.getTime() <= 0 ) {
            return Util.EMPTY_STRING;
        }
        return itsUIExtDateFormat.format(theDate);
    }

    public static String FormatExtYearDateTime( java.util.Date theDate ) {
        
        if ( theDate == null ) { 
            return Util.EMPTY_STRING;
        }
        if ( theDate.getTime() <= 0 ) {
            return Util.EMPTY_STRING;
        }
        return itsUIExtYearDateTimeFormat.format(theDate);
    }

    public static String FormatNormalDateTime( java.util.Date theDate ) {

        if ( theDate == null ) { 
            return Util.EMPTY_STRING;
        }
        // com.aeon23.compsplus.cpdebug.out("formatting normal date time " + theDate + " (" + theDate.getTime() + ") = " + itsUIDateTimeFormat.format(theDate));

        return itsUIDateTimeFormat.format(theDate);
    }

    public static String FormatNormalDateTime( String theDate ) {
        
        if ( theDate == null ) { 
            return Util.EMPTY_STRING;
        }
        try {
            return itsUIDateTimeFormat.format(itsSQLDateTimeFormat.parse(theDate));
        } catch (Exception e) {
            return Util.EMPTY_STRING;
        }
    }

    public static String FormatNormalTimestamp( Timestamp theTS ) {
        if ( theTS == null ) {
            return Util.EMPTY_STRING;
        }
        try {
            return itsUIDateTimeFormat.format(new Date(theTS.getTime()));
        } catch ( Exception e ) {
            return Util.EMPTY_STRING;
        }
    }

    public static String FormatNormalTime( long theTime ) {
        try {
            return itsUITimeFormat.format(new Date(theTime));
        } catch ( Exception e ) {
            return Util.EMPTY_STRING;
        }
    }

    public static String FormatRelativeTimestamp( Timestamp theTS ) {
        if ( theTS == null ) {
            return Util.EMPTY_STRING;
        }
        try {
            Date aTSDate = new Date(theTS.getTime());
            if ( Util.getTodayDate(aTSDate).getTime() == Util.getTodayDate(new Date()).getTime() ) {
                return itsUITimeFormat.format(new Date(theTS.getTime()));
            } else {
                return itsUIDateTimeFormat.format(new Date(theTS.getTime()));
            }
        } catch ( Exception e ) {
            return Util.EMPTY_STRING;
        }
    }

    public static String FormatRelativeTimestampEx( Timestamp theTS ) {
        if ( theTS == null ) {
            return Util.EMPTY_STRING;
        }
        return FormatRelativeTimeEx( theTS.getTime() );
    }
    
    public static String FormatRelativeTimeEx ( long theMillis ) {
        try {
            Date aTSDate = new Date(theMillis);
            if ( Util.getTodayDate(aTSDate).getTime() == Util.getTodayDate(new Date()).getTime() ) {
                return itsUITimeFormat.format(aTSDate);
            } else {
                return itsUIExtDateTimeFormat.format(aTSDate);
            }
        } catch ( Exception e ) {
            return Util.EMPTY_STRING;
        }
    }

    public static String FormatDateTimeEx ( long theMillis ) {
        try {
            return itsUIExtDateTimeFormat.format(new Date(theMillis));
        } catch ( Exception e ) {
            return Util.EMPTY_STRING;
        }
    }

    public static String FormatUICurrency( String theValue ) {

        java.text.NumberFormat aFormat = java.text.NumberFormat.getCurrencyInstance();
        try {
            return aFormat.format( Double.parseDouble(theValue) );
        } catch ( Exception e ) {
            return Util.EMPTY_STRING;
        }

    }

    public static String FormatUICurrency( Double theValue ) {

        java.text.NumberFormat aFormat = java.text.NumberFormat.getCurrencyInstance();
        try {
            return aFormat.format( theValue.doubleValue() );
        } catch ( Exception e ) {
            return Util.EMPTY_STRING;
        }

    }

    public static Long ParseSQLLong( ResultSet theRS, int i ) throws SQLException {
        long aValue = theRS.getLong(i);
        if ( aValue == 0 && theRS.wasNull() ) {
            return null;
        }
        return new Long(aValue);
    }

    public static Double ParseSQLDouble( ResultSet theRS, int i ) throws SQLException {
        double aValue = theRS.getDouble(i);
        if ( aValue == 0 && theRS.wasNull() ) {
            return null;
        }
        return new Double(aValue);
    }

    public static java.util.Date ParseUIDate( String theDate ) throws DBException {
        java.util.Date aDate = null;
        if ( theDate == null || theDate.equals(Util.EMPTY_STRING) ) {
            return null;
        }
        try { 
            aDate = itsUIDateFormat.parse(theDate);
        } catch ( Exception e ) {
            throw new DBException( e, theDate + " is not a valid date (1) (use " + itsUIDateFormat.toPattern() + ")");
        }
        if ( aDate == null ) {
            throw new DBException( null, theDate + " is not a valid date (2) (use " + itsUIDateFormat.toPattern() + ")");
        }
        return aDate;
    }

    public static java.util.Date ParseUIDateTime( String theDate ) throws DBException {
        java.util.Date aDate = null;
        if ( theDate == null || theDate.equals(Util.EMPTY_STRING) ) {
            return null;
        }
        try { 
            aDate = itsUIDateTimeFormat.parse(theDate);
        } catch ( Exception e ) {
            throw new DBException( e, theDate + " is not a valid date-time (1) (use " + itsUIDateTimeFormat.toPattern() + ")");
        }
        if ( aDate == null ) {
            throw new DBException( null, theDate + " is not a valid date-time (2) (use " + itsUIDateTimeFormat.toPattern() + ")");
        }
        return aDate;
    }

    public static Long ParseJavaLong ( String theLong ) throws DBException {
        // com.aeon23.compsplus.cpdebug.out("Parsing long " + theLong);
        Long aLong = null;
        try { 
            aLong = new Long(theLong);
        } catch ( Exception e ) {
            return null;
            // throw new DBException( e, theLong + " is not a valid number");
        }
        return aLong;
    }

    public static double ParseJavaDouble ( String theDouble ) throws DBException {
        double aDouble = -1.0;
        try { 
            aDouble = Double.parseDouble(theDouble);
        } catch ( Exception e ) {
            return 0;
            // throw new DBException( e, theDouble + " is not a valid number");
        }
        return aDouble;
    }

    public static long ParseCheckbox ( String theCheckbox ) throws DBException {
        if ( theCheckbox == null || theCheckbox.equals(Util.EMPTY_STRING) ) {
            return 0;
        }
        return 1;
    }

    public static String validateState( String theValue ) {
        int stop = STATES.length;
        for ( int i=0; i<stop; i++ ) {
            if ( theValue.toUpperCase().equals(STATES[i].toUpperCase()) ) {
                return theValue.toUpperCase();
            }
        }
        return null;
    }

    public static String validateZIP( String theValue ) {
        String aZIP  = DBFormat.stripCharsFromString( theValue, "- " );
        int stop = aZIP.length();
        
        if ( stop != 5 && stop != 9 ) {
            return null;
        }
        for ( int i=0; i<stop; i++ ) {
            if ( ! Character.isDigit(aZIP.charAt(i)) ) {
                return null;
            }
        }
        if ( stop == 5 ) {
            return aZIP;
        } else {
            return aZIP.substring(0,6) + "-" + aZIP.substring(6,10);
        }
    }

    public static String validatePhone( String theValue ) {
        String aPhone = DBFormat.stripCharsFromString( theValue, "()- " );
        int stop = aPhone.length();
        if ( stop != 10 ) {
            return null;
        }
        for ( int i=0; i<stop; i++ ) {
            if ( ! Character.isDigit(aPhone.charAt(i)) ) {
                return null;
            }
        }
        return aPhone.substring(0,3) 
            + "-" + aPhone.substring(3,7)
            + "-" + aPhone.substring(7,10);
    }

    public static String validateEmail( String theValue ) {
        int stop = theValue.length();
        boolean atFound = false;
        boolean dotFound = false;
        char c;

        for ( int i=0; i<stop; i++ ) {
            c = theValue.charAt(i);
            if ( c == '@' ) {
                if ( i == 0 || i == stop-1 ) {
                    return null;
                }
                atFound = true;
            }
            if ( atFound && c == '.' ) {
                if ( i == stop-1 ) {
                    return null;
                }
                dotFound = true;
            }
        }

        if ( atFound && dotFound ) {
            return theValue;
        }
        return null;
    }

    public static String UnformatUICurrency( String theString ) {
        return stripCharsFromString(theString, "$,");
    }

    private static String stripCharsFromString( String theString, String theChars ) {

        if ( theString == null ) return null;
        if ( theChars == null ) return theString;

        int stop = theString.length();
        int aCharLength = theChars.length();
        String rstr = Util.EMPTY_STRING;
        boolean foundChar = false;

        for ( int i=0; i<stop; i++ ) {
            char c = theString.charAt(i);
            for ( int j=0; j<aCharLength; j++ ) {
                if ( c == theChars.charAt(j) ) {
                    foundChar = true;
                    break;
                }
            }
            if ( !foundChar ) {
                rstr += c;
            }
            foundChar = false;
        }
        return rstr;
    }

    private static String replaceCharInString( String theString, char theChar, String theRepl ) {

        int stop = theString.length();
        String rstr = Util.EMPTY_STRING;
        boolean foundChar = false;

        for ( int i=0; i<stop; i++ ) {
            char c = theString.charAt(i);
            if ( c == theChar ) {
                foundChar = true;
                rstr += theRepl;
            }
            if ( !foundChar ) {
                rstr += c;
            }
            foundChar = false;
        }
        return rstr;
    }

    public static String FormatSQLLikeString( String s ) {
        return "%" + DBFormat.replaceCharInString( s, ' ', "%" ) + "%";
    }

    public static String getHashValueNZ( Map theHash, String theKey ) {
        if ( theHash != null ) {
            Object o = theHash.get(theKey);
            if ( o != null ) {
                if ( o instanceof String ) {
                    return (String) o;
                }
                return o.toString();
            }
        }
        return Util.EMPTY_STRING;
    }

    public static String FormatStartTime ( int theMins ) {
        String rval = Util.EMPTY_STRING;
        rval += String.valueOf( theMins / 60 ) 
            + ":"
            + String.valueOf( theMins - ((theMins/60)*60) )
            ;
        if ( rval.indexOf(":") == 1 ) rval = "0" + rval;
        if ( rval.endsWith(":0") ) rval += "0";
        return rval;
    }

    public static int ParseStartTime ( String theStartTime ) {
        int aColonPos = theStartTime.indexOf(":");
        if ( aColonPos == -1 ) return -1;

        return ( (60 * Integer.parseInt(theStartTime.substring(0,aColonPos)))
                 + Integer.parseInt(theStartTime.substring(aColonPos+1)) );
    }

    public static String FormatDuration ( long theDelta ) {
        long aNumHours = theDelta / Util.MILLIS_IN_HOUR;
        long aNumMins = (theDelta - (aNumHours*Util.MILLIS_IN_HOUR)) / Util.MILLIS_IN_MINUTE;
        return FormatDuration(aNumHours, aNumMins);
    }

    public static String FormatDuration( long theNumHours, long theNumMins ) {
        String rstr = Util.EMPTY_STRING;
        if ( theNumMins >= 59 ) {
            theNumMins = 0; theNumHours++;
        }
        if ( theNumHours > 0 ) {
            rstr = String.valueOf(theNumHours) + " hour";
            if ( theNumHours > 1 ) rstr += "s";
            if ( theNumMins > 0 ) rstr += ", ";
        }
        if ( theNumMins > 0 ) {
            rstr += String.valueOf(theNumMins) + " minute";
            if ( theNumMins > 1 ) rstr += "s";
        }
        return rstr;
    }

}
