/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/JSPUtil.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: JSPUtil.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.util.Enumeration;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.tagext.*;

public class JSPUtil {

    public static final String FONT_BODYTEXT_START = "<font class=\"bodytext\">";
    public static final String FONT_HEADER2_START = "<font class=\"header2\">";
    public static final String FONT_HEADER3_START = "<font class=\"header3\">";
    public static final String FONT_END = "</font>";
    public static final String STR_REQUIRED = "<font color=\"#ff0000\">*</font>";

    public static final VariableInfo[] EMPTY_VARINFO = new VariableInfo[] {};

    public static void sendRedirect( HttpServletResponse response, String theURL ) {
        try {
            response.sendRedirect(theURL);

        } catch ( Exception e ) {
            System.err.println("JSPUtil.sendRedirect: " + e.toString());
            e.printStackTrace();
        }
    }

    public static String findCookie( HttpServletRequest theRequest,
                                     String theCookieName ) {
        if ( theRequest == null ) {
            return null;
        }

        Cookie[] someCookies = theRequest.getCookies();
        Cookie aCookie = null;
        int stop = someCookies.length;
        for ( int i=0; i<stop; i++ ) {
            aCookie = someCookies[i];
            if ( aCookie != null && aCookie.getName().equals(theCookieName) ) {
                return aCookie.getValue();
            }
        }

        return null;
    }

    public static String appendAllParams( String theURL, ServletRequest request ) {

        Enumeration somePNames = request.getParameterNames();
        String rstr = theURL;

        if ( theURL.indexOf("?") == -1) rstr += "?";
        else if ( !theURL.endsWith("?") ) rstr += "&";

        boolean isFirst = false;
        String aPName = null;
        while ( somePNames.hasMoreElements() ) {
            if ( !isFirst ) rstr += "&";
            isFirst = false;
            aPName = (String) somePNames.nextElement();
            rstr += aPName + "=" + java.net.URLEncoder.encode(request.getParameter(aPName));
        }

        return rstr;
    }

    public static String getFullURL ( HttpServletRequest request ) {
        String aBase = request.getRequestURI();
        String aQuery = request.getQueryString();
        return (aQuery == null) ? aBase : aBase + "?" + aQuery;
    }

    public static final void unCacheableHeader( HttpServletResponse response ) {
        // response.setHeader("max-age", "0");
        response.setHeader("Cache-Control", "max-age=0, must-revalidate, no-cache");
        // response.setHeader("Expires", "Fri, 30 Oct 1998 14:19:41 GMT");
    
	}
	
	// this to prevent the web browser from caching
	public static String unCacheableMeta() {
		return "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n";
	}

    private JSPUtil () {}

}
