/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/Mailer.java,v 1.1.1.1 2001/09/29 20:38:48 jonathan Exp $
 * $Log: Mailer.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.util.Map;
import java.util.Iterator;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import javax.mail.internet.InternetAddress;
import org.opendj.util.Util;
import org.opendj.util.template.TemplateMailer;
import org.opendj.util.log.Logger;

public class Mailer {

    public static final int M_REGISTRATION        =  0;
    public static final int M_REQUEST_SUB         =  1;
    public static final int M_UNREQUEST_SUB       =  2;
    public static final int M_BE_SUB              =  3;
    public static final int M_NEW_WEEKLY_SHOW     =  4;
    public static final int M_NEW_SINGLE_SHOW     =  5;
    public static final int M_NEW_PASSWORD        =  6;
    public static final int M_EMAIL_CHANGED       =  7;
    public static final int M_SPECIFY_RERUN       =  8;
    public static final int M_RERUN_REMOVED       =  9;
    public static final int M_CHANGE_RERUN        = 10;

    private static String M_TEMPLATE_DIR = null;
    public static final String M_TEMPLATE_SUFFIX = ".mt";
    public static final String[] M_TEMPLATES = {
        "registration",             //  0
        "requestSub",               //  1
        "unrequestSub",             //  2
        "beSub",                    //  3
        "newWeeklyShow",            //  4
        "newSingleShow",            //  5
        "newPassword",              //  6
        "emailChanged",             //  7
        "specifyRerun",             //  8
        "rerunRemoved",             //  9
        "changeRerun",              // 10
        null
    };
    public static final String[] M_ADMIN_TEMPLATES = {
        "registration_admin",       //  0
        "requestSub_admin",         //  1
        "unrequestSub_admin",       //  2
        "beSub_admin",              //  3
        "newWeeklyShow_admin",      //  4
        "newSingleShow_admin",      //  5
        "newPassword_admin",        //  6
        "emailChanged_admin",       //  7
        "specifyRerun_admin",       //  8
        "rerunRemoved_admin",       //  9
        "changeRerun_admin",        // 10
        null
    };
    public static final String BASE_SYMBOLS_FILE = M_TEMPLATE_DIR + "globalsyms";

    private static InternetAddress GENERIC_FROM_EMAIL = null;
    private static InternetAddress ADMIN_FROM_EMAIL = null;
    private static InternetAddress SITEOWNER_EMAIL = null;
    private static InternetAddress WEBMASTER_EMAIL = null;

    private synchronized static void init() {
        if ( ADMIN_FROM_EMAIL == null ) {
            try {
                Config aConfig = Config.global();
                M_TEMPLATE_DIR = aConfig.getProperty(Config.PROP_MAILTEMPLATEDIR);
                GENERIC_FROM_EMAIL
                    = new InternetAddress(aConfig.getProperty(Config.PROP_FROMEMAIL),
                                          aConfig.getProperty(Config.PROP_FROMNAME));
                ADMIN_FROM_EMAIL
                    = new InternetAddress(aConfig.getProperty(Config.PROP_ADMINFROMEMAIL),
                                          aConfig.getProperty(Config.PROP_ADMINFROMNAME));
                SITEOWNER_EMAIL
                    = new InternetAddress(aConfig.getProperty(Config.PROP_ADMINTOEMAIL),
                                          aConfig.getProperty(Config.PROP_ADMINTONAME));
                WEBMASTER_EMAIL
                    = new InternetAddress(aConfig.getProperty(Config.PROP_WEBMTOEMAIL),
                                          aConfig.getProperty(Config.PROP_WEBMTONAME));
            } catch ( Exception e ) {
                Logger.global().capture("Mailer.init", e, Logger.L_FIRE);
            }
        }
    }

    public static InternetAddress getFromEmail() {
        init();
        return GENERIC_FROM_EMAIL;
    }

    public static void queue ( InternetAddress theFrom, 
                               InternetAddress theTo,
                               int theTemplate ) {
        Mailer.queue(theFrom, theTo, theTemplate, null);
    }

    public static void queue ( InternetAddress theFrom, 
                               InternetAddress theTo,
                               int theTemplate,
                               Map theSymbols ) {
        
        String aTemplateFile = null;

        // Send "regular" email
        if ( M_TEMPLATES[theTemplate] != null ) {
            aTemplateFile = M_TEMPLATE_DIR + M_TEMPLATES[theTemplate] + M_TEMPLATE_SUFFIX;
            queue_INTERNAL(theFrom, theTo, aTemplateFile, theSymbols);
        }

        // Send "admin" email
        if ( M_ADMIN_TEMPLATES[theTemplate] != null ) {
            init();
            theFrom = ADMIN_FROM_EMAIL;
            theTo = getAdminToEmail( theTemplate );
            aTemplateFile = M_TEMPLATE_DIR + M_ADMIN_TEMPLATES[theTemplate] + M_TEMPLATE_SUFFIX;
            queue_INTERNAL(theFrom, theTo, aTemplateFile, theSymbols);
        }
    }
    
    private static void queue_INTERNAL( InternetAddress theFrom,
                                        InternetAddress theTo,
                                        String theTemplateFile,
                                        Map theSymbols ) {
        try {
            TemplateMailer.queue(theFrom, 
                                 theTo, 
                                 theTemplateFile, 
                                 /*BASE_SYMBOLS_FILE*/ null, 
                                 theSymbols);
            
        } catch ( Exception e ) {
            Logger.global().capture("Mailer.queue_INTERNAL(from:" + theFrom 
                                    + ",to:" + theTo
                                    + ",template:" + theTemplateFile + ")", 
                                    e, 
                                    Logger.L_MAILER);
        }
    }

    public static InternetAddress getAdminToEmail ( int theTemplate ) {
        switch ( theTemplate ) {
            /*
        case M_LOGMESSAGE:
            return WEBMASTER_EMAIL;
            */
        default:
            return SITEOWNER_EMAIL;
        }
    }

    public static void main ( String[] args ) {
        if ( args.length == 0 ) {
            printUsage();
            return;
        }
        try {
            String aParam = args[0];
            if ( aParam.equals("gensyms") ) {
                regenerateSymbolsFile();

            } else if ( aParam.equals("wait") ) {
                Util.sleep(Long.parseLong(args[1]));

            } else {
                // Unknown option
                printUsage();
            }

        } catch ( Exception e ) {
            System.err.println("Error in Mailer.main: " + e.toString());
            e.printStackTrace();
        }
    }

    private static void regenerateSymbolsFile() throws IOException {
        /*
        PrintStream aFileOut = new PrintStream(new FileOutputStream(BASE_SYMBOLS_FILE));
        Iterator aGlobalIterator = DBGlobalManager.global().getIterator();
        String aGName  = null;
        String aGValue = null;

        while ( aGlobalIterator.hasNext() ) {
            aGName  = (String) aGlobalIterator.next();
            aGValue = (String) DBGlobalManager.global().get(aGName);
            aFileOut.println(aGName + "=" + aGValue);
        }

        // Write any additional custom global symbols here...

        aFileOut.close();
        */
    }

    public static void printUsage() {
        System.out.println("Usage:\n\nMailer gensyms");
    }
}
