/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/TimePair.java,v 1.1.1.1 2001/09/29 20:38:48 jonathan Exp $
 * $Log: TimePair.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.util.Date;

public class TimePair {

    public long start = -1;
    public long end = -1;
    public int special = S_NONE;
    public String specialStart = null;
    public String specialEnd = null;

    public TimePair () {}

    public boolean isFatal() {
        return ( special == S_NONEXISTENT || 
                 special == S_CANNOT_START_IN_DT2ST_REPEAT_HOUR ||
                 special == S_CANNOT_END_IN_DT2ST_REPEAT_HOUR );
    }

    public String formatMessage () {
        String rstr = Util.EMPTY_STRING;
        if ( special == S_NONE ) return Util.EMPTY_STRING;
        rstr += "Please note the following about the time slot that will start at ";
        if ( specialStart != null ) {
            rstr += specialStart + " and end at " + specialEnd;
        } else {
            rstr += DBFormat.FormatNormalDate(new Date(start)) 
                + " and end at "
                + DBFormat.FormatNormalDate(new Date(end))
                ;
        }
        rstr += ":<br>" + ERR_MESSAGES[special] + "<br>";
        return rstr;
    }

    public static final int S_NONE                              = 0;
    public static final int S_NONEXISTENT                       = 1;
    public static final int S_DT_START_ADVANCED_TO_3            = 2;
    public static final int S_DT_HOUR_SKIPPED                   = 3;
    public static final int S_CANNOT_START_IN_DT2ST_REPEAT_HOUR = 4;
    public static final int S_CANNOT_END_IN_DT2ST_REPEAT_HOUR   = 5;
    public static final int S_ST_EXTRA_HOUR                     = 6;
    public static final int S_DT_END_ADVANCED_TO_3              = 7;
    public static final int S_DT_JUST_STARTED                   = 8;
    public static final int S_ST_JUST_STARTED                   = 9;

    public static final String[] ERR_MESSAGES
        = { "The time slot is OK.",

            "The time slot cannot exist because it is entirely within 2:00AM to 3:00AM on the "
            + "first Sunday of April. "
            + "Due to Daylight Saving Time, the hour between 2:00AM and 3:00AM is skipped.",

            "The time slot is OK, but the start time will be moved to 3:00AM, because the start "
            + "time is between 2:00AM and 3:00AM on the first Sunday of April. "
            + "Due to Daylight Saving Time, the hour between 2:00AM and 3:00AM is skipped.",

            "The time slot is OK, but will actually be one hour shorter than it appears, because "
            + "it occupies the hour between 2:00AM and 3:00AM on the first Sunday of April. "
            + "Due to Daylight Saving Time, the hour between 2:00AM and 3:00AM is skipped.",

            "The time slot cannot exist because the start time is between 1:00AM and 2:00AM on the "
            + "last Sunday of October.  "
            + "Due to Daylight Saving Time, the hour between 1:00AM and 2:00AM is repeated.",

            "The time slot cannot exist because the end time is between 1:00AM and 2:00AM on the "
            + "last Sunday of October.  "
            + "Due to Daylight Saving Time, the hour between 1:00AM and 2:00AM is repeated.",

            "The time slot is OK, but will actually be one hour longer than it appears, because "
            + "it occupies the hour between 1:00AM and 2:00AM on the last Sunday of October. "
            + "Due to Daylight Saving Time, the hour between 1:00AM and 2:00AM is repeated.",

            "The time slot is OK, but will end time has been moved to 3:00AM (PDT), because the end "
            + "time that you specified was between 2:00AM and 3:00AM on the first Sunday of April. "
            + "Due to Daylight Saving Time, the hour between 2:00AM and 3:00AM is skipped."
            + "Note that even though it seems like your show will be longer, it is actually "
            + "shorter than it should be because the hour between 2:00AM and 3:00AM is skipped.",

            "The time slot is OK, but Daylight Saving Time begins at 2:00AM, so the hour between "
            + "2:00AM and 3:00AM is skipped.  Please make sure you set your clock correctly the day "
            + "before so you won't miss your show.",

            "The time slot is OK, but Daylight Saving Time ends at 2:00AM, so the hour between "
            + "1:00AM and 2:00AM is repeated.  Please make sure you set your clock correctly the day "
            + "before so you will arrive for your show on time."
        };
}
