/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/TimeUtil.java,v 1.1.1.1 2001/09/29 20:38:49 jonathan Exp $
 * $Log: TimeUtil.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.util.Enumeration;
import java.util.List;
import java.util.Vector;
import java.util.Calendar;
import java.util.Date;
import javax.servlet.http.*;

public class TimeUtil {

    public static final boolean DST_ACTIVE = true;

    public static final String DT_TZ = "PDT";
    public static final String ST_TZ = "PST";

    public static boolean isDST () {
        return (Calendar.getInstance().get(Calendar.DST_OFFSET) > 0);
    }

    public static boolean isDST ( long theTimeVal ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date(theTimeVal));
        return (cal.get(Calendar.DST_OFFSET) > 0);
    }

    public static boolean isDST ( Date theDate ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(theDate);
        return (cal.get(Calendar.DST_OFFSET) > 0);
    }

    public static boolean isDSTStart ( long theTimeVal ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date(theTimeVal));
        return ( cal.get(Calendar.DAY_OF_WEEK)  == Calendar.SUNDAY &&
                 cal.get(Calendar.MONTH)        == Calendar.APRIL &&
                 cal.get(Calendar.DAY_OF_MONTH) <= 7 );
    }

    public static boolean isDSTEnd ( long theTimeVal ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date(theTimeVal));
        return ( cal.get(Calendar.DAY_OF_WEEK)  == Calendar.SUNDAY &&
                 cal.get(Calendar.MONTH)        == Calendar.OCTOBER &&
                 cal.get(Calendar.DAY_OF_MONTH) >= 25 );
    }

    public static TimePair DSTPolicy ( Date theDate, int theStartMins, int theEndMins ) {

        theDate = Util.getTodayDate(theDate);

        TimePair rval = new TimePair();
        rval.start = theDate.getTime() + (Util.MILLIS_IN_MINUTE*theStartMins);
        rval.end = theDate.getTime() + (Util.MILLIS_IN_MINUTE*theEndMins);
        rval.special = TimePair.S_NONE;

        int aDay = Util.getDay(theDate);
        int aMonth = Util.getMonth(theDate);
        int aDayOfMonth = Util.getDayOfMonth(theDate);
        int aStartHour = theStartMins / 60;
        int aStartMinute = theStartMins % 60;
        int anEndHour = theEndMins / 60;
        int anEndMinute = theEndMins % 60;

        debug.out("DSTPolicy("+theDate+","+theStartMins+","+theEndMins+")\n"
                  + "aDay=" + aDay + "\n"
                  + "aMonth=" + aMonth + "\n"
                  + "aDayOfMonth=" + aDayOfMonth + "\n"
                  + "aStartHour=" + aStartHour + "\n"
                  + "aStartMinute=" + aStartMinute + "\n"
                  + "anEndHour=" + anEndHour + "\n"
                  + "anEndMinute=" + anEndMinute + "\n"
                  );

        Calendar aStartCal = Calendar.getInstance();
        aStartCal.setTime(theDate);

        Calendar anEndCal = Calendar.getInstance();
        anEndCal.setTime(theDate);

        if ( aDay        == 0 && 
             aMonth      == 3 &&
             aDayOfMonth <= 7 ) {

            // First Sunday in April, DST starts...
            debug.out("first sunday in april");

            if ( aStartHour == 2 && anEndHour > 2 ) {
                debug.out("start hour is 2, endhour > 2");
                // Make sure the end hour is not 3am exactly, in which case
                // this slot should not exist
                if ( anEndHour == 3 && anEndMinute == 0 ) {
                    debug.out("end hour is 3, endmins is 0");
                    rval.start = -1; rval.end = -1;
                    rval.special = TimePair.S_NONEXISTENT;
                    rval.specialStart = DBFormat.FormatNormalDate(theDate) + " 02:00";
                    rval.specialEnd   = DBFormat.FormatNormalDate(theDate) + " 03:00";
                    return rval;
                }
                // The show starts at 2 but ends after 3, so change
                // the start time to be 3am PDT and the end time to be PDT
                aStartCal.set(Calendar.DST_OFFSET, (int) Util.MILLIS_IN_HOUR);
                aStartCal.set(Calendar.HOUR_OF_DAY, 3);
                anEndCal.set(Calendar.DST_OFFSET, (int) Util.MILLIS_IN_HOUR);
                anEndCal.set(Calendar.HOUR_OF_DAY, anEndHour);
                anEndCal.set(Calendar.MINUTE, anEndMinute);
                rval.start   = aStartCal.getTime().getTime();
                rval.end     = anEndCal.getTime().getTime();
                rval.special = TimePair.S_DT_START_ADVANCED_TO_3;
                rval.specialStart = DBFormat.FormatNormalDate(theDate) + " " 
                    + DBFormat.FormatStartTime(theStartMins);
                rval.specialEnd   = DBFormat.FormatNormalDate(theDate) + " "
                    + DBFormat.FormatStartTime(theEndMins);
                return rval;

            } else if ( aStartHour < 2 && anEndHour == 2 ) {
                // Ends within the "dead zone", so change end hour to be 3am PDT
                debug.out("starthour < 2 and endhour is 2");
                aStartCal.set(Calendar.DST_OFFSET, 0);
                aStartCal.set(Calendar.HOUR_OF_DAY, aStartHour);
                aStartCal.set(Calendar.MINUTE, aStartMinute);
                anEndCal.set(Calendar.DST_OFFSET, (int) Util.MILLIS_IN_HOUR);
                anEndCal.set(Calendar.HOUR_OF_DAY, 3);
                anEndCal.set(Calendar.MINUTE, 0);
                rval.start   = aStartCal.getTime().getTime();
                rval.end     = anEndCal.getTime().getTime();
                rval.special = TimePair.S_DT_END_ADVANCED_TO_3;
                rval.specialStart = DBFormat.FormatNormalDate(theDate) + " " 
                    + DBFormat.FormatStartTime(theStartMins);
                rval.specialEnd   = DBFormat.FormatNormalDate(theDate) + " "
                    + DBFormat.FormatStartTime(theEndMins);
                return rval;

            } else if ( aStartHour < 2 && anEndHour > 2 ) {
                debug.out("starthour < 2, endhour > 2");
                // Spans the DST border - so force start time to be PST,
                // and force end time to be PDT
                aStartCal.set(Calendar.DST_OFFSET, 0);
                aStartCal.set(Calendar.HOUR_OF_DAY, aStartHour);
                aStartCal.set(Calendar.MINUTE, aStartMinute);
                anEndCal.set(Calendar.DST_OFFSET, (int) Util.MILLIS_IN_HOUR);
                anEndCal.set(Calendar.HOUR_OF_DAY, anEndHour);
                anEndCal.set(Calendar.MINUTE, anEndMinute);
                rval.start = aStartCal.getTime().getTime();
                rval.end   = anEndCal.getTime().getTime();
                rval.special = TimePair.S_DT_HOUR_SKIPPED;
                rval.specialStart = DBFormat.FormatNormalDate(theDate) + " " 
                    + DBFormat.FormatStartTime(theStartMins);
                rval.specialEnd   = DBFormat.FormatNormalDate(theDate) + " "
                    + DBFormat.FormatStartTime(theEndMins);
                return rval;

            } else if ( aStartHour > 2 ) {
                debug.out("starthour > 2");
                // Force start and end times to be PDT
                aStartCal.set(Calendar.DST_OFFSET, (int) Util.MILLIS_IN_HOUR);
                aStartCal.set(Calendar.HOUR_OF_DAY, aStartHour);
                aStartCal.set(Calendar.MINUTE, aStartMinute);
                anEndCal.set(Calendar.DST_OFFSET, (int) Util.MILLIS_IN_HOUR);
                anEndCal.set(Calendar.HOUR_OF_DAY, anEndHour);
                anEndCal.set(Calendar.MINUTE, anEndMinute);
                rval.start = aStartCal.getTime().getTime();
                rval.end   = anEndCal.getTime().getTime();
                rval.specialStart = DBFormat.FormatNormalDate(theDate) + " " 
                    + DBFormat.FormatStartTime(theStartMins);
                rval.specialEnd   = DBFormat.FormatNormalDate(theDate) + " "
                    + DBFormat.FormatStartTime(theEndMins);
                rval.special = TimePair.S_DT_JUST_STARTED;
                return rval;
            }

        } else if ( aDay        == 0  &&
                    aMonth      == 9  &&
                    aDayOfMonth >= 25 ) {

            // Last Sunday in October, DST ends...

            // Handle illegal cases first
            // Cannot start or end between 1am and 2am
            if ( aStartHour == 1 && aStartMinute != 0 ) {
                rval.start = -1; rval.end = -1;
                rval.special = TimePair.S_CANNOT_START_IN_DT2ST_REPEAT_HOUR;
                rval.specialStart = DBFormat.FormatNormalDate(theDate) 
                    + " " + DBFormat.FormatStartTime(theStartMins);
                rval.specialEnd = DBFormat.FormatNormalDate(theDate) 
                    + " " + DBFormat.FormatStartTime(theEndMins);
                return rval;
            }
            if ( anEndHour == 1 && anEndMinute != 0 ) {
                rval.start = -1; rval.end = -1;
                rval.special = TimePair.S_CANNOT_END_IN_DT2ST_REPEAT_HOUR;
                rval.specialStart = DBFormat.FormatNormalDate(theDate) 
                    + " " + DBFormat.FormatStartTime(theStartMins);
                rval.specialEnd = DBFormat.FormatNormalDate(theDate) 
                    + " " + DBFormat.FormatStartTime(theEndMins);
                return rval;
            }

            if ( aStartHour <= 2 ) {
                // Starts at or before 2am, set the start time to PDT
                aStartCal.set(Calendar.DST_OFFSET, (int) Util.MILLIS_IN_HOUR);
                aStartCal.set(Calendar.HOUR_OF_DAY, aStartHour);
                aStartCal.set(Calendar.MINUTE, aStartMinute);
                rval.start = aStartCal.getTime().getTime();
            }
            if ( anEndHour >= 2 ) {
                // Ends at or after 2am, so make sure the end time is PST
                anEndCal.set(Calendar.DST_OFFSET, 0);
                anEndCal.set(Calendar.HOUR_OF_DAY, anEndHour);
                anEndCal.set(Calendar.MINUTE, anEndMinute);
                rval.end   = anEndCal.getTime().getTime();
                rval.special = TimePair.S_ST_JUST_STARTED;
            }
            // If it crosses the 1am-2am border completely, mark the special attr
            if ( aStartHour <= 1 && anEndHour > 1 ) {
                rval.special = TimePair.S_ST_EXTRA_HOUR;
            }
            return rval;

        }
        return rval;
    }

    public static TimePair checkSingleShow ( Date theDate, int theStartMins, int theEndMins ) {
        return DSTPolicy(theDate, theStartMins, theEndMins);
    }

    public static List checkWeeklyShow ( int theDay, 
                                         int theStartMins, 
                                         int theEndMins,
                                         Date theStartLease,
                                         Date theEndLease ) {
        long aStart = -1;
        long anEnd = -1;
        List rList = new Vector();

        // lease ends at the end of the endlease date
        long anEndLeaseMillis = theEndLease.getTime() + Util.MILLIS_IN_DAY; 
        
        // Find the first show here...
        Date aBroadcastDate = Util.getNextDay( theDay, theStartLease );

        while ( aBroadcastDate.getTime() < anEndLeaseMillis ) {
            aBroadcastDate = Util.getTodayDate(aBroadcastDate);

            rList.add(DSTPolicy(aBroadcastDate, theStartMins, theEndMins));

            aBroadcastDate = Util.getNextDay( theDay, 
                                              new Date(aBroadcastDate.getTime()
                                                       + (2*Util.MILLIS_IN_DAY) ) );
        }
        return rList;
    }

    public static String appendTimePairToURL ( String theURL, TimePair tp ) {
        int tpidx = 0;
        if ( tp.special != TimePair.S_NONE ) {
            theURL
                += "&tpstart" + tpidx + "=" + tp.start
                + "&tpend" + tpidx + "=" + tp.end
                + "&tpstat" + tpidx + "=" + tp.special
                ;
            if ( tp.specialStart != null ) {
                theURL += "&tpscs" + tpidx + "=" + java.net.URLEncoder.encode(tp.specialStart);
            }
            if ( tp.specialEnd != null ) {
                theURL += "&tpsce" + tpidx + "=" + java.net.URLEncoder.encode(tp.specialEnd);
            }
        }
        return theURL;
    }

    public static String appendTimePairsToURL ( String theURL, List theTimePairs ) {
        int stop=theTimePairs.size();
        TimePair tp = null;
        int tpidx = 0;
        for (int i=0; i<stop; i++) {
            tp = (TimePair) theTimePairs.get(i);
            if ( tp.special != TimePair.S_NONE ) {
                theURL
                    += "&tpstart" + tpidx + "=" + tp.start
                    + "&tpend" + tpidx + "=" + tp.end
                    + "&tpstat" + tpidx + "=" + tp.special
                    ;
                if ( tp.specialStart != null ) {
                    theURL += "&tpscs" + tpidx + "=" + java.net.URLEncoder.encode(tp.specialStart);
                }
                if ( tp.specialEnd != null ) {
                    theURL += "&tpsce" + tpidx + "=" + java.net.URLEncoder.encode(tp.specialEnd);
                }
            }
        }
        return theURL;
    }
    
    public static boolean getURLHasInfo ( HttpServletRequest request ) {
        return request.getQueryString().indexOf("&tpstart") != -1;
    }

    public static String getMessage ( HttpServletRequest request ) {

        Enumeration somePNames = request.getParameterNames();
        String aPName = null;
        TimePair[] someTimePairs = new TimePair[4];
        int anIndex = -1;
        int aMaxIndex = -1;

        while ( somePNames.hasMoreElements() ) {
            aPName = (String) somePNames.nextElement();
            // debug.out("TimeUtil found PName='" + aPName + "'");
            if ( aPName.startsWith("tpstart") ) {
                try {
                    anIndex = Integer.parseInt(aPName.substring(aPName.length()-1));
                    if ( someTimePairs[anIndex] == null ) {
                        someTimePairs[anIndex] = new TimePair();
                    }
                    someTimePairs[anIndex].start = Long.parseLong(request.getParameter(aPName));
                } catch ( Exception e ) { 
                    e.printStackTrace();
                    someTimePairs[anIndex].start = -1;
                    continue;
                }
            } else if ( aPName.startsWith("tpend") ) {
                try {
                    anIndex = Integer.parseInt(aPName.substring(aPName.length()-1));
                    if ( someTimePairs[anIndex] == null ) {
                        someTimePairs[anIndex] = new TimePair();
                    }
                    someTimePairs[anIndex].end = Long.parseLong(request.getParameter(aPName));
                } catch ( Exception e ) { 
                    e.printStackTrace();
                    someTimePairs[anIndex].end = -1;
                    continue;
                }

            } else if ( aPName.startsWith("tpstat") ) {
                try {
                    anIndex = Integer.parseInt(aPName.substring(aPName.length()-1));
                    if ( someTimePairs[anIndex] == null ) {
                        someTimePairs[anIndex] = new TimePair();
                    }
                    someTimePairs[anIndex].special = Integer.parseInt(request.getParameter(aPName));
                } catch ( Exception e ) { 
                    e.printStackTrace();
                    someTimePairs[anIndex].special = -1;
                    continue;
                }
            } else if ( aPName.startsWith("tpscs") ) {
                try {
                    anIndex = Integer.parseInt(aPName.substring(aPName.length()-1));
                    if ( someTimePairs[anIndex] == null ) {
                        someTimePairs[anIndex] = new TimePair();
                    }
                    someTimePairs[anIndex].specialStart = request.getParameter(aPName);
                } catch ( Exception e ) { 
                    e.printStackTrace();
                    someTimePairs[anIndex].specialStart = null;
                    continue;
                }
            } else if ( aPName.startsWith("tpsce") ) {
                try {
                    anIndex = Integer.parseInt(aPName.substring(aPName.length()-1));
                    if ( someTimePairs[anIndex] == null ) {
                        someTimePairs[anIndex] = new TimePair();
                    }
                    someTimePairs[anIndex].specialEnd = request.getParameter(aPName);
                } catch ( Exception e ) { 
                    e.printStackTrace();
                    someTimePairs[anIndex].specialEnd = null;
                    continue;
                }
            }
            if ( anIndex > aMaxIndex ) aMaxIndex = anIndex;
        }

        String rstr = Util.EMPTY_STRING;
        TimePair tp = null;
        String aTPMessage = null;
        for ( int i=0; i<=aMaxIndex && i<someTimePairs.length; i++ ) {
            tp = someTimePairs[i];
            if ( tp == null || tp.start < 0 || tp.end < 0 || tp.special < 0 ) {
                debug.out("tp was bad...");
                // continue;
            }
            try {
                aTPMessage = tp.formatMessage();
            } catch ( Exception e ) {
                // XXX huh?
                e.printStackTrace();
                aTPMessage = Util.EMPTY_STRING;
            }
            rstr += aTPMessage;
        }

        return rstr;
    }

    private TimeUtil() {}

}
