/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/Util.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: Util.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.ByteArrayOutputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.net.URLEncoder;
import java.net.InetAddress;
import java.security.MessageDigest;
import java.util.List;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Date;
import java.util.Calendar;
import java.util.Properties;
import java.util.Enumeration;
import java.util.zip.CRC32;
import javax.mail.internet.InternetAddress;

/**
 * Supplies several static utility methods used by other classes
 *
 * @author Jonathan Cobb, 2000
 * @version 0.1
 */
public class Util extends org.opendj.share.Util {

    public static final int FIELD_IDENTIFIER = 0;
    public static final int FIELD_PASSWORD   = 1;
    public static final int FIELD_TEXT       = 2;
    public static final int FIELD_TEXTAREA   = 3;
    public static final int FIELD_SELECT     = 4;

    public static final int FIELD_REQUIRED    = 0;
    public static final int FIELD_NOTREQUIRED = 1;

    public static final String EMPTY_STRING = "";

    public static final int MIN_PASSWORD_LENGTH = 12;

    public static final String[] DAY_NAMES = { "Sun", "Mon", "Tue", "Wed", 
                                               "Thu", "Fri", "Sat" };

    public static final String[] DAY_NAMES_FULL = { "Sunday", "Monday", 
                                                    "Tuesday", "Wednesday", 
                                                    "Thursday", "Friday", 
                                                    "Saturday" };

    public static final String MD5_SEPARATOR = "_";

    /**
     * Splices a String into a List.
     *
     * @param s the source string
     * @param sep a string which separates elements in the source string
     * @return a List of Strings, or null if s was null
     */
    public static List stringToList( String s, String sep ) {

        if ( s == null ) return null;

        StringTokenizer st = new StringTokenizer(s, sep);
        List aList = new Vector();

        while ( st.hasMoreTokens() ) {
            aList.add(st.nextToken());
        }

        return aList;
    }

    /**
     * Splices a String into an array of Strings.
     *
     * @param s the source string
     * @param sep a string which separates elements in the source string
     * @return an array of Strings, or null if s was null or empty
     */
    public static String[] stringToArray( String s, String sep ) {

        if ( s == null ) return null;

        List     aList       = Util.stringToList(s, sep);
        int      aNumStrings = aList.size();

        // If the list is empty, or only contains one element that is null or empty,
        // we return null.
        if ( aNumStrings == 0 || 
             (aNumStrings == 1 && 
              ( ((String) aList.get(0))          == null ||
                ((String) aList.get(0)).length() == 0 ) ) ) {
            return null;
        }

        String[] anArray     = new String[aNumStrings];
        Iterator anIterator  = aList.iterator();

        for ( int i=0; anIterator.hasNext(); i++ ) {
            anArray[i] = (String) anIterator.next();
        }
        return anArray;
    }

    /**
     * Outputs all mappings in a Map as a string
     *
     * @param theMap the map to output
     * @param theMappingSeparator a String to place in between mappings
     * @param theMappingOperator a String to place between a key and its value
     * @return a String representing all the mappings in the Map
     */
    public static String mapToString( Map theMap, String theMappingSeparator, String theMappingOperator ) {
        return mapToString( theMap, theMappingSeparator, theMappingOperator, MAP_NOENCODE );
    }

    public static final int MAP_NOENCODE  = 0;
    public static final int MAP_URLENCODE = 1;
    /**
     * Outputs all mappings in a Map as a string
     *
     * @param theMap the map to output
     * @param theMappingSeparator a String to place in between mappings
     * @param theMappingOperator a String to place between a key and its value
     * @param theEncodeType if MAP_URLENCODE, the values are URLEncoded
     * @return a String representing all the mappings in the Map
     */
    public static String mapToString( Map theMap, 
                                      String theMappingSeparator, 
                                      String theMappingOperator,
                                      int theEncodeType ) {
        Iterator i = theMap.keySet().iterator();
        String rstr = "";
        Object aKey = null;
        Object aValue = null;

        while ( i.hasNext() ) {
            aKey = i.next();
            aValue = theMap.get(aKey);
            if ( rstr.length() > 0 ) {
                rstr += theMappingSeparator;
            }
            if ( aValue == null ) {
                if ( theEncodeType == MAP_URLENCODE ) {
                    rstr += aKey.toString() + theMappingOperator + "";
                } else {
                    rstr += aKey.toString() + theMappingOperator + "null";
                }
            } else {
                if ( theEncodeType == MAP_URLENCODE ) {
                    rstr += aKey.toString() + theMappingOperator + URLEncoder.encode(aValue.toString());
                } else {
                    rstr += aKey.toString() + theMappingOperator + aValue.toString();
                }
            }
        }

        return rstr;
    }

    /**
     * Outputs all mappings in a Map as a string
     *
     * @param theMap the map to output
     * @return a String representing all the mappings in the Map
     */
    public static String mapToString( Map theMap ) {
        return mapToString(theMap, "|", "=>", MAP_NOENCODE);
    }

    /**
     * Strip the last file extension from the given file
     * Simply removes anything after the last dot in the filename, including the dot
     *
     * @param theFileName the file name whose extension will be removed
     * @return the new filename, with the last extension removed
     */
    public static String stripExtension( String theFileName ) {
        if ( theFileName == null ) return null;
        int aDotPos = theFileName.lastIndexOf(".");
        if ( aDotPos == -1 ) return theFileName;
        return theFileName.substring(0, aDotPos);
    }

    public static final int NO_CREATE_DIRS = 0;
    public static final int CREATE_DIRS    = 1;
    
    /**
     * Returns a File object for a given directory name
     *
     * @param theDirName the directory name
     * @param theCreateFlag if this is CREATE_DIRS, the directory (and any parents that also do not exist)
     *                      will be created if it does not exist.  
     *                      if it is NO_CREATE_DIRS, an IOException will be thrown if it does not exist. 
     * @return a File object for the directory
     * @exception IOException if the name does not refer to a directory or 
     *                        if the name refers to a non-existant directory and theCreateFlag was false
     */
    public static File getDirectory ( String theDirName, int theCreateFlag ) throws IOException {
        File anOutputDir = new File(theDirName);
        if ( !anOutputDir.isDirectory() ) {
            throw new IOException( theDirName + " is not a directory" );
        }
        if ( !anOutputDir.exists() ) {
            if ( theCreateFlag == CREATE_DIRS ) {
                if ( !anOutputDir.mkdirs() ) {
                    throw new IOException( "Could not create directory " + theDirName );
                }

            } else {
                throw new IOException( "Directory " + theDirName + " does not exist" );
            }
        }
        return anOutputDir;
    }

    /**
     * Sleeps for the specified period of time, EVEN IF INTERRUPTED.
     * All interrupts are ignored.  The thread, if awoken, will go back
     * to sleep for the remainder of the time left to sleep.
     * Be VERY careful when using this method -- ideally you should not
     * go into a deep sleep for more than a few seconds.
     *
     * @param ms the number of milliseconds to sleep
     */
    public static void deepSleep( long ms ) {
        long aStart = System.currentTimeMillis();
        long aNowTime = System.currentTimeMillis();
        while ( aNowTime < aStart + ms ) {
            try {
                Thread.sleep(ms - (aNowTime - aStart));
            } catch (InterruptedException e) {}
            aNowTime = System.currentTimeMillis();
        }
    }

    /**
     * Copy a file from src to dest.
     *
     * @param src the source file to copy
     * @param the destination file
     * @exception IOException if an exception occurred copying the file
     */
    public static void copyFile ( File src, File dest ) throws IOException {

        FileInputStream  fin = null;
        FileOutputStream fout = null;

        try {
            debug.out("starting copy...");
            fin  = new FileInputStream(src);
            fout = new FileOutputStream(dest);

            Util.copyStream( fin, fout );

        } catch ( Exception e ) {
            debug.out("Caught exception: " + e.toString());
            e.printStackTrace();

        } finally {
            try { fin.close();  } catch ( Exception e2 ) {}
            try { fout.close(); } catch ( Exception e2 ) {}
        }
        debug.out("done.");
    }

    public static final String TEMPDIR = "\\Temp";

    /**
     * Creates a temporary file.
     *
     * @param prefix the prefix for the temp file
     * @param suffix the suffix for the temp file
     * @return a File object corresponding to the new temp file
     * @exception IOException if an error occurs while trying to create the file
     */
    public static File createTempFile ( String prefix, String suffix ) throws IOException {
        return File.createTempFile(prefix,suffix);
        /*
        return new File(TEMPDIR 
                        + File.separator 
                        + prefix 
                        + System.currentTimeMillis() 
                        + "_" 
                        + Math.random() 
                        + suffix);
        */
    }


    /**
     * Appends a value to a list held within a map.  If the list does not
     * exist, it is created and theValue is inserted as the first element.
     *
     * @param theMap the map to find the list in.
     * @param theKey the key in the map that points to the list
     * @param theValue the value to append to the list
     */
    public static void addToValueList ( Map theMap, String theKey, String theValue ) {
        List aList = (List) theMap.get(theKey);
        if ( aList == null ) {
            aList = new Vector();
            theMap.put(theKey, aList);
        }
        aList.add(theValue);
    }

    /**
     * Given a string to match and an array of strings, returns the index in the
     * array where the string was found, or -1 if not found.  Note that the comparison
     * is CASE INSENSITIVE
     *
     * @param theString the String to search for in the array
     * @param theStringArray the array of Strings to search
     * @return the index of theString in the theStringArray, or -1 if it could not be found.
     */
    public static int getStringIndexCI ( String theString, String[] theStringArray ) {
        for ( int i=0; i<theStringArray.length; i++ ) {
            if ( theStringArray[i] != null && theStringArray[i].equalsIgnoreCase(theString) ) {
                return i;
            }
        }
        return -1;
    }


    /**
     * Given a full class name, return the package name.
     * Essentially, all this does is chop text after the last period (including the last period).
     *
     * @param theClassName the fully-package-qualitfied name of the class
     * @return the name of the package.
     */
    public static String getPackageName ( String theClassName ) {
        return stripLast(theClassName, ".");
    }
    
    /**
     * Given a full class name, return the simple name.
     * Essentially, all this does is chop all text before the last period (including the last period).
     *
     * @param theClassName the fully-package-qualitfied name of the class
     * @return the simple name of the class.
     */
    public static String getClassName ( String theClassName ) {
        return stripUpTo(theClassName, ".");
    }

    /**
     * This method removes the last occurrence of theDelim from theString, and
     * any text that occurs after theDelim as well
     *
     * @param theString the source String
     * @param theDelim the delimiter text
     * @return a modified String, with theDelim and any subsequent text removed.
     */
    public static String stripLast ( String theString, String theDelim ) {
        if ( theString == null || theDelim == null ) return theString;
        int aDelimPos = theString.lastIndexOf(theDelim);
        if ( aDelimPos == -1 ) return theString;
        return theString.substring(0,aDelimPos);
    }

    /**
     * Formats a String for presentation in a source code file.
     * 
     * @param s the source String
     * @return "s" (in quotes) is returned is s is not null.  If s is null, null is returned.
     */
    public static String sourceCodeString ( String s ) {
        return ( s == null ) ? "null" : "\"" + s + "\"";
    }

    /**
     * @param n the number of tabs
     * @return a String composed of ( 4 x n ) space characters
     */
    public static String tab ( int n ) {
        String rstr = "";
        for ( int i=0; i<n; i++ ) rstr += "    ";
        return rstr;
    }

    /**
     * @param theArray an array of Objects
     * @return a String representing all elements of the array, each separated by ",".  
     *         If theArray is null, the method returns null.
     */
    public static String arrayToString( Object[] theArray ) {
        return arrayToString(theArray, ",");
    }

    /**
     * @param theArray an array of Objects
     * @param theDelim a String to use as a delimiter in composing the return string
     * @return a String representing all elements of the array, each separated by theDelim.  
     *         If theArray or theDelim is null, the method returns null.
     */
    public static String arrayToString( Object[] theArray, String theDelim ) {
        if ( theArray == null || theDelim == null ) return null;
        String rstr = "";
        for ( int i=0; i<theArray.length; i++ ) {
            if ( i > 0 ) {
                rstr += theDelim;
            }
            if ( theArray[i] != null ) {
                rstr += theArray[i].toString();
            } else {
                rstr += "null";
            }
        }
        return rstr;
    }

    /**
     * @param theArray an array of ints
     * @param theDelim a String to use as a delimiter in composing the return string
     * @return a String representing all elements of the array, each separated by theDelim.  
     *         If theArray or theDelim is null, the method returns null.
     */
    public static String arrayToString( int[] theArray, String theDelim ) {
        if ( theArray == null || theDelim == null ) return null;
        String rstr = "";
        for ( int i=0; i<theArray.length; i++ ) {
            if ( i > 0 ) {
                rstr += theDelim;
            }
            rstr += theArray[i];
        }
        return rstr;
    }

    /**
     * @param theList a List of Strings (or other objects that have valid toString methods)
     * @param theDelim a String to use as a delimiter in composing the return string
     * @return a String representing all elements of the List, each separated by theDelim.  
     *         If theList or theDelim is null, the method returns null.
     */
    public static String listToString( List theList, String theDelim ) {
        if ( theList == null || theDelim == null ) return null;
        String rstr = "";
        Iterator anIterator = theList.iterator();
        for ( int i=0; anIterator.hasNext(); i++ ) {
            if ( i > 0 ) {
                rstr += theDelim;
            }
            rstr += anIterator.next().toString();
        }
        return rstr;
    }

    /**
     * Determine if a string contains any whitespace chars.
     *
     * @param s a string to analyze
     * @return true if theString contains whitespace, false otherwise
     */
    public static boolean containsWhitespace ( String s ) {
        if ( s == null ) return false;
        int len = s.length();
        for ( int i=0; i<len; i++ ) {
            if ( Character.isWhitespace(s.charAt(i)) ) return true;
        }
        return false;
    }

    /**
     * Calculate the CRC32 for a byte array.
     *
     * @param theBytes the bytes to calculate the CRC32 for
     * @return the CRC32 of the bytes.
     */
    public static long getCRC32 ( byte[] theBytes ) {
        CRC32 crc = new CRC32();
        crc.reset();
        crc.update(theBytes);
        return crc.getValue();
    }

    /**
     * Calculate the MD5 digest for a byte array.
     *
     * @param theBytes the bytes to calculate the CRC32 for
     * @return the MD5 of the bytes as a byte array.
     */
    public static byte[] getMD5 ( byte[] theBytes ) {
        try {
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update( theBytes );
            return md5.digest();
        } catch ( Exception e ) {
            // Doh!
            System.err.println("Util.getMD5: " + e.toString() + "\n" + Util.getStackTrace(e));
            return null;
        }
    }

    /**
     * Read all the bytes from a stream and return them as a byte array.
     *
     * @param theInStream the InputStream to read.
     * @return a byte array of all the data from the input stream.
     * @exception IOException if an IO error occurs.
     */
    public static byte[] readStreamAsByteArray ( InputStream theInStream ) throws IOException {
        ByteArrayOutputStream anOutStream = null;
        anOutStream = new ByteArrayOutputStream(DEFAULT_BUFFER_SIZE);
        Util.copyStream(theInStream, anOutStream);
        return anOutStream.toByteArray();
    }

    /**
     * Makes all necesary parent directories for a file.
     *
     * @param theFileName the name of the file whose parent directories are to be made.
     * @exception IOException if any of the directories cannot be made.
     *                        Note that even if this exception is thrown, some of the directories may
     *                        have already been created.
     */
    public static void makeDirsForFile ( String theFileName ) throws IOException {

        String aDirName = Util.stripLast(theFileName, File.separator);
        File aDir = new File(aDirName);
        
        // Only create the dir if it does not exist
        if ( !aDir.exists() ) {
            if ( !aDir.mkdirs() ) {
                throw new IOException("Util.makeDirsForFile: could not make all directories.");
            }
        }
    }

    /**
     * Create a Map where only those keys in srcMap that start with prefix
     * are included.  The prefix is stripped from the key when it is inserted
     * into the return Map.  This method operates only on keys that are Strings.
     * All other keys are ignored.
     *
     * @param srcMap the source Map
     * @param prefix the key prefix.
     * @return a Map that includes only key-value pairs from srcMap where the key starts with prefix
     */
    public static Map subMap ( Map srcMap, String prefix ) {
        if ( srcMap == null ) return null;

        Map destMap = new HashMap();
        if ( prefix == null ) {
            return destMap;
        }

        Iterator someKeys = srcMap.keySet().iterator();
        Object key = null;
        String keyString = null;
        int prefixLength = prefix.length();

        while ( someKeys.hasNext() ) {
            key = someKeys.next();
            if ( key instanceof String ) {
                keyString = (String) key;
                if ( keyString != null && keyString.startsWith(prefix) ) {
                    destMap.put(keyString.substring(prefixLength), srcMap.get(key));
                }
            }
        }

        return destMap;
    }

    /**
     * Parse a String into a Map.
     * 
     * @param srcString the source String to parse.
     * @param destMap the destination map to place mappings into.  If this is null, a new Map will be created.
     * @param mappingSep the String which separates mappings from one another in the srcString
     * @param keyValSep the String which separates a key from its corresponding value
     * @param prefix if not null, this indicates a prefix to prepend to all key names found in srcString.
     * @return the destMap
     */
    public static Map stringToMap ( String srcString,
                                    Map    destMap,
                                    String mappingSep,
                                    String keyValSep,
                                    String prefix ) {

        if ( destMap == null ) destMap = new HashMap();
        if ( prefix  == null ) prefix = "";

        StringTokenizer st = new StringTokenizer(srcString, mappingSep);
        String aMapping = null;
        int aKeyValSepLen = keyValSep.length();
        int aKeyValSepPos = -1;

        while ( st.hasMoreTokens() ) {
            aMapping = st.nextToken();
            aKeyValSepPos = aMapping.indexOf(keyValSep);
            if ( aKeyValSepPos != -1 ) {
                destMap.put(prefix + aMapping.substring(0, aKeyValSepPos), aMapping.substring(aKeyValSepPos + aKeyValSepLen));
            } else {
                // If no keyValSep found, insert the key with a null value
                destMap.put(prefix + aMapping, null);
            }
        }

        return destMap;
    }

    /**
     * Returns a Date value for the very start of a given date.
     *
     * @param theDate any valid Date object
     * @return a Date object corresponding to the very beginning of theDate (00:00:00)
     */
    public static Date getTodayDate ( Date theDate ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(theDate);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }

    /**
     * Returns a Date value for the day previous to a given date.
     *
     * @param theDate any valid Date object
     * @return a Date object corresponding to one day before theDate (00:00:00)
     */
    public static Date getYesterdayDate ( Date theDate ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(theDate);
        cal.add(Calendar.DATE, -1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }

    /**
     * Returns a Date value for the day after a given date.
     *
     * @param theDate any valid Date object
     * @return a Date object corresponding one day after theDate (00:00:00)
     */
    public static Date getTomorrowDate ( Date theDate ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(theDate);
        cal.add(Calendar.DATE, 1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }

    /**
     * Count the number of alphanumeric characters in a string
     *
     * @param s any string
     * @return the number of alphanumeric characters in the string
     */
    public static int countAlphanumericChars( String s ) {
        int len = s.length();
        char c;
        int rval = 0;
        for ( int i=0; i<len; i++ ) {
            c = s.charAt(i);
            if ( (c >= 'a' && c <= 'z') ||
                 (c >= 'A' && c <= 'Z') ||
                 (c >= '0' && c <= '9') ) {
                rval++;
            }
        }
        return rval;
    }

    /**
     * Given a Date, returns the numeric day of the week, Sunday=0
     */
    public static int getDay ( Date theDate ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(theDate);
        return cal.get(Calendar.DAY_OF_WEEK) - 1;
    }

    /**
     * Given a Date, returns the numeric number of the month, January=0
     */
    public static int getMonth ( Date theDate ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(theDate);
        return cal.get(Calendar.MONTH);
    }

    /**
     * Given a Date, returns the day of the month, between 1 and 31
     */
    public static int getDayOfMonth ( Date theDate ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(theDate);
        return cal.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * Given a day and a Date, return the Date that corresponds to the 
     * next occurence of day.  This could return the same date passed in.
     */
    public static Date getNextDay( int theDay, Date theDate ) {
        Date aStartDate = Util.getTodayDate(theDate);
        int aDay = Util.getDay(aStartDate);

        if ( aDay == theDay ) {
            // It's today
            return aStartDate;

        } else if ( theDay > aDay ) {
            // theDay is later in this week
            return new Date( aStartDate.getTime() + (Util.MILLIS_IN_DAY*(theDay-aDay)) );
        } else {
            // theDay is next week
            return new Date( aStartDate.getTime() + (Util.MILLIS_IN_DAY*(theDay-aDay+7)) );
        }
    }

    /**
     * Returns true if the specified bit is set
     */
    public static boolean checkBit( int theBitfield, int theBit ) {
        return ( theBitfield & theBit ) != 0;
    }

    /**
     * Given a millisecond time value, return the largest
     * time value that is an on-the-hour time and is less than millis
     *
     * @param millis a millisecond time value
     * @return a time value that is the largest on-the-hour time that is less than millis
     */
    public static long floorToHour ( long millis ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date(millis));
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime().getTime();
    }

    /**
     * Returns the String value of an object.  If the object is null, returns EMPTY_STRING.
     *
     * @param o the object
     * @return o.toString() if o is not null, EMPTY_STRING otherwise
     */
    public static String nzo ( Object o ) {
        if ( o == null ) return EMPTY_STRING;
        return o.toString();
    }

    /**
     * @return the local IP address, in dotted-decimal form.
     */
    public static String getLocalHostAddr() {
        try {
            return InetAddress.getLocalHost().getHostAddress();
        } catch ( Exception e ) {
            // XXX Whoa
            return null;
        }
    }

    /**
     * @returns an InternetAddress, or null if an exception occurred
     */
    public static InternetAddress getInternetEmail ( String theEmail, String theName ) {
        try {
            return new InternetAddress(theEmail, theName);
        } catch ( Exception e ) {
            // Argh!
            return null;
        }
    }

    /**
     * Given a String, returns a String that can be used as a valid file name.
     * Mostly, this just replaces non-alphanumeric characters with underscores.
     *
     * @param s the String
     * @return a String that may be used as a filename.
     */
    public static String makeSafeFileName ( String s ) {
        if ( s == null ) return null;
        int len = s.length();
        char c;
        String rstr = Util.EMPTY_STRING;
        for ( int i=0; i<len; i++ ) {
            c = s.charAt(i);
            if ( Character.isLetterOrDigit(c) ) {
                rstr += c;
            } else {
                rstr += "_";
            }
        }
        return rstr;
    }

    /**
     * Get the MD5 for a string, but return the raw bytes
     * Base64 encoded.
     *
     * @param theString a String to calculate an MD5 for
     * @return the MD5 as a Base64-encoded string
     */
    public static String getMD5Base64 ( String theString ) {
        return Base64.encode(Util.getMD5(theString.getBytes()));
    }

    /**
     * Get the MD5 for a String, but instead of formatting the result
     * as raw bytes, format the result as an ASCII (UTF-8) String.
     *
     * @param theString a string to calculate an MD5 for
     * @return the MD5 as a formatted String
     */
    public static String getMD5Text ( String theString ) {
        try {
            if ( theString == null ) return null;
            byte[] bytes = Util.getMD5(theString.getBytes());
            String rstr = Util.EMPTY_STRING;
            for ( int i=0; i<bytes.length; i++ ) {
                if ( i > 0 ) rstr += MD5_SEPARATOR;
                rstr += String.valueOf(bytes[i]);
            }
            debug.out("Util.getMD5Text: len=" + rstr.length() + ", rstr='" + rstr + "'");
            return rstr;

        } catch ( Exception e ) {
            // Doh!
            System.err.println("Util.getMD5Text: " + e.toString() + "\n" + Util.getStackTrace(e));
            return null;
        }
    }

    public static final String[] HEX_DIGITS = { "0", "1", "2", "3", 
                                                "4", "5", "6", "7",
                                                "8", "9", "a", "b", 
                                                "c", "d", "e", "f" };
    /**
     * Get the hexadecimal string representation for a byte.
     * The leading 0x is not included.
     *
     * @param b the byte to process
     * @return a String representing the hexadecimal value of the byte
     */
    public static String getHexValue ( byte b ) {
        int i = (int)b + 128; // now  0 <= i <= 255
        return HEX_DIGITS[i/16] + HEX_DIGITS[i-(16*(i/16))];
    }

    /**
     * Given a String of hex values separated by hyphens, convert it to an MD5 String
     *
     * @param s the source String
     * @return an MD5 String
     */
    public static String stringToMD5 ( String s ) {
        if ( s == null ) return null;
        int len = s.length();
        List someHexValues = Util.stringToList(s, MD5_SEPARATOR);
        Iterator i = someHexValues.iterator();
        byte[] bytes = new byte[64];
        String aHexVal = null;
        int aCount = 0;
        while ( i.hasNext() && aCount < bytes.length ) {
            aHexVal = (String) i.next();
            // debug.out("Translating a HexVal='" + aHexVal + "'...");
            bytes[aCount] = (byte) Byte.parseByte(aHexVal);
            aCount++;
        }
        return new String(bytes, 0, aCount);
    }
    
    /**
     * Never returns.
     */
    public static void spin () {
        while ( true ) { Util.sleep(2000); }
    }

    /**
     * All methods in this class are static, so the constructor is private.
     * This class should never be instantiated.
     */
    private Util() {}
}
