/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/log/Logger.java,v 1.1.1.1 2001/09/29 20:38:49 jonathan Exp $
 * $Log: Logger.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util.log;

import java.sql.*;
import java.io.*;
import java.net.InetAddress;

import javax.mail.*;
import javax.mail.internet.*;

import org.opendj.util.*;
import org.opendj.util.mail.*;

public class Logger {

    public final static int L_LEGACY   = -2;
    public final static int L_MAILER   = -1;
    public final static int L_DEBUG    = 0;
    public final static int L_INFO     = 1;
    public final static int L_DATABASE = 2;
    public final static int L_WARNING  = 3;
    public final static int L_ERROR    = 4;
    public final static int L_FIRE     = 5;
    public final static int L_FATAL    = 6;

    // Must be 0 or greater, DO NOT SET TO -1 or lower
    private static int L_ALERTLEVEL = 3;

    private static String GLOBAL_LOGDIR   = null;
    public static final String DEFAULT_LOGFILE = "opendj_tomcat";

    private FileWriter LOG = null;

    private static Logger GLOBAL_LOG = null;
    private static String GLOBAL_NAME = null;

    private String itsFromEmail = null;
    private String itsToEmail = null;

    public static void initGlobal( String theLogDir, String theLogName ) {
        GLOBAL_LOGDIR = theLogDir;
        GLOBAL_NAME = theLogName;
    }

    public static synchronized Logger global() {
        if ( GLOBAL_LOG == null ) {

            if ( GLOBAL_NAME == null ) GLOBAL_NAME = DEFAULT_LOGFILE;
            GLOBAL_LOG = new Logger(GLOBAL_NAME);
        }

        return GLOBAL_LOG;
    }

    public Logger ( String theLogName ) {
        try {
            if ( GLOBAL_LOGDIR == null ) {
                GLOBAL_LOGDIR = Config.global().getProperty(Config.PROP_LOGDIR);
            }
            // open in append mode
            LOG = new FileWriter(GLOBAL_LOGDIR + theLogName + ".log", true);
        } catch ( Exception e ) {
            // SHIT
            panic("Logger.constructor", e);
        }
        itsFromEmail = Config.global().getProperty(Config.PROP_LOGGERFROMEMAIL);
        itsToEmail   = Config.global().getProperty(Config.PROP_LOGGERTOEMAIL);
        if ( itsFromEmail == null || itsFromEmail.length() == 0 ||
             itsToEmail == null || itsToEmail.length() == 0 ) {
            panic("FROM/TO EMAILS NOT SETUP PROPERLY!!!", null);
            L_ALERTLEVEL = Integer.MAX_VALUE;
        }
    }
    
    public void capture( String theMessage,
                         Throwable theT,
                         int theLevel ) {
        
        // Generate detailed error message
        InetAddress localaddr = null;
        String addrString = "";
        String hostname = null;
        try {
            localaddr = InetAddress.getLocalHost();
            addrString = localaddr.toString();
            hostname = localaddr.getHostName();
        } catch ( Exception e ) {
            localaddr = null;
            addrString = "[InetAddress.getLocalHost generated exception: " 
                + e.toString() + "]";
            hostname = "unknown";
        }
        String aLevelName = Logger.getLevelName(theLevel);
        String msg = "\n" + (new java.util.Date()).toString()
            + "\nLogger on " + hostname
            + "\nhost=" + addrString
            + "\nreports " + aLevelName
            + " condition:\n'" + theMessage + "'";
        if ( theT == null ) {
            theT = new Exception("NO-EXCEPTION-OCCURRED");
            msg += "\n\nNo exception was given."
                + "\n\nStack Trace:\n"
                + getStackTrace(theT);
        } else {
            msg += "\n\nException: '" + theT.toString() 
                + "'\n\nStack Trace:\n"
                + getStackTrace(theT);
        }

        // Write to log
        writeLog(msg);

        // Send mail
        if ( theLevel >= L_ALERTLEVEL ) {
            try {
                MailPiece mp = new MailPiece();
                mp.fromEmail = new InternetAddress(itsFromEmail);
                mp.toEmails = new InternetAddress[1];
                mp.toEmails[0] = new InternetAddress(itsToEmail);
                mp.subject = "Logger (" + hostname + "): " + aLevelName;
                mp.message = msg;
                MailManager.global().queue( mp );
            } catch ( Exception e ) {
                writeLog("\n*** ERROR QUEUEING MAIL: " 
                         + e.toString()
                         + "\nStack Trace:\n" + getStackTrace(e));
            }
        }
    }

    private synchronized void writeLog( String msg ) {
        try {
            LOG.write("\n----------------------------------------"
                      + msg);
            LOG.flush();
        } catch ( Exception e ) {
            // SHIT.  THIS HAD BETTER NOT FAIL
            panic("----------ERROR---------- Logger failed "
                  + "writing to log file!", e);
        }
    }

    public static String getLevelName( int theLevel ) {
        switch ( theLevel ) {
        case L_MAILER:
            return "MAILER";
        case L_DEBUG:
            return "DEBUG";
        case L_INFO:
            return "INFO";
        case L_WARNING:
            return "WARNING";
        case L_ERROR:
            return "ERROR";
        case L_FIRE:
            return "FIRE";
        case L_FATAL:
            return "FATAL";
        case L_LEGACY:
            return "LEGACY";
        case L_DATABASE:
            return "DATABASE";
        default:
            return "UNKNOWN";
        }
    }

    public static String getStackTrace( Throwable theT ) {
        try {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            theT.printStackTrace(pw);
            String rval = sw.toString();
            pw.close();
            if ( theT instanceof NestedException && 
                 ((NestedException) theT).getBaseThrowable() != null ) {
                rval += "\nNESTED STACK TRACE:\n"
                     + getStackTrace( ((NestedException)theT).getBaseThrowable() );
            }
            return rval;
        } catch ( Exception e ) {
            return "\n\nLogger.getStackTrace GENERATED EXCEPTION: '" 
                + e.toString() + "'\n\n";
        }
    }

    public static void panic( String s ) {
        System.err.println("\n------------------>Logger.panic! "
                           + "Message='" + s + "'"
                           + "\n\n");
    }

    public static void panic( String s, Throwable t ) {
        System.err.println("\n------------------>Logger.panic! "
                           + "Message='" + s + "', "
                           + "Exception='" + t.toString() + "', "
                           + "Stack Trace:" + getStackTrace(t)
                           + "\n\n");
    }

}
