/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/service/ServiceListener.java,v 1.1.1.1 2001/09/29 20:38:49 jonathan Exp $
 * $Log: ServiceListener.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util.service;

import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.StringTokenizer;
import org.opendj.util.log.Logger;

public class ServiceListener implements Runnable {
    
    private volatile Thread itsMainThread = null;

    private ServerSocket itsServerSocket = null;
    private int itsPort = -1;
    private Socket itsConnection = null;
    private PrintWriter itsOutWriter = null;
    private BufferedReader itsInReader = null;
    
    private ServiceManager itsManager = null;

    private volatile boolean isListenerDone = false;

    private boolean isAuthenticated = false;
    private String itsAuthPassword = null;
    
    public ServiceListener (ServiceManager theManager, 
                            int thePort,
                            String theAuthPassword ) {
        itsManager = theManager;
        itsPort = thePort;
        itsAuthPassword = theAuthPassword;
    }
    
    public void go () {
        Thread aThread = new Thread(this);
        aThread.start();
    }
    
    public void run () {
        itsMainThread = Thread.currentThread();
        try {
            itsServerSocket = new ServerSocket( itsPort );
            while ( !isListenerDone ) {
                itsConnection = itsServerSocket.accept();
                if ( !handleConnection(itsConnection) ) return;
                isAuthenticated = false;
            }
        } catch ( Exception e ) {
            itsManager.log("ServiceListener.run: unexpected exception", e, Logger.L_ERROR);
        } finally {
            closeAll();
        }
    }

    public void finish () {
        isListenerDone = true;
        closeAll();
        if ( itsMainThread != null && itsMainThread.isAlive() ) {
            itsMainThread.interrupt();
            try { itsMainThread.join(); } catch ( Exception e ) {}
        }
    }

    private void closeAll() {
        try { itsConnection.close(); } catch ( Exception e ) {}
        try { itsServerSocket.close(); } catch ( Exception e ) {}
    }

    public boolean handleConnection (Socket theSocket) {
        isAuthenticated = false;
        try {
            itsOutWriter = new PrintWriter( new OutputStreamWriter( theSocket.getOutputStream() ),
                                            true ); // with autoflush
            itsInReader = new BufferedReader( new InputStreamReader( theSocket.getInputStream() ) );
            String aLine = null;
            StringTokenizer st = null;
            String aCommand = null;
            String anArg = null;

            while ( true ) {
                aLine = itsInReader.readLine();
                st = new StringTokenizer(aLine);
                if ( st.hasMoreTokens() ) {
                    aCommand = st.nextToken().toLowerCase();
                } else {
                    itsOutWriter.println("?");
                    continue;
                }
                if ( aCommand.equals("quit") ) {
                    itsOutWriter.println("BYE");
                    return true;
                }
                if ( st.hasMoreTokens() ) {
                    anArg = st.nextToken();
                } else {
                    itsOutWriter.println("?");
                    continue;
                }
                if ( !isAuthenticated ) {
                    if ( aCommand.equals("auth") &&
                         anArg.equals(itsAuthPassword) ) {
                        isAuthenticated = true;
                        itsOutWriter.println("AUTH");
                    } else {
                        itsManager.log("ServiceListener: auth failed from " 
                                       + theSocket.getInetAddress(), null, Logger.L_WARNING);
                        break;
                    }
                } else {
                    if ( aCommand.equals("start") ) {
                        if ( itsManager.initService(anArg) != null ) {
                            itsOutWriter.println("START");
                        } else {
                            itsOutWriter.println("ERRORSTARTING");
                        }

                    } else if ( aCommand.equals("stop") ) {
                        if ( itsManager.stopService(anArg) ) {
                            itsOutWriter.println("STOP");
                        } else {
                            itsOutWriter.println("ERRORSTOPPING");
                        }
                        if ( anArg.equals("ALL") ) return false;

                    } else if ( aCommand.equals("list") ) {
                        itsManager.printRunningServices(itsOutWriter);
                    }
                }
            }

        } catch ( Exception e ) {
            itsManager.log("ServiceListener.handleConnection", e, Logger.L_ERROR);

        } finally {
            try { theSocket.close(); } catch ( Exception e ) {}
        }
        return true;
    }
}
