/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/template/TemplateMailer.java,v 1.1.1.1 2001/09/29 20:38:49 jonathan Exp $
 * $Log: TemplateMailer.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util.template;

import java.io.IOException;
import java.util.Map;
import javax.mail.internet.InternetAddress;

import org.opendj.util.mail.*;

public class TemplateMailer {

    public static final String HEADER_SUBJECT = "Subject:";

    public static void queue ( InternetAddress theFrom, 
                               InternetAddress theTo,
                               String theTemplateFile,
                               String theSymbolsFile ) 
        throws IOException, MailException, TemplateParserException {

        TemplateMailer.queue(theFrom, theTo, theTemplateFile, theSymbolsFile, null);
    }

    public static void queue ( InternetAddress theFrom, 
                               InternetAddress theTo,
                               String theTemplateFile,
                               String theSymbolsFile,
                               Map theSymbols ) 
        throws IOException, MailException, TemplateParserException {

        String aMessage = "";
        String aSubject = "";

        // Build message from template
        TemplateParserReader aReader = new TemplateParserReader( theTemplateFile,
                                                                 theSymbolsFile,
                                                                 theSymbols );
        String aLine = aReader.readLine();

        // EMPTY MESSAGE
        if ( aLine == null ) return;

        // First line could be Subject header
        if ( aLine.startsWith(HEADER_SUBJECT) ) {
            aSubject = aLine.substring(HEADER_SUBJECT.length()).trim();
            aLine = aReader.readLine();
        } else {
            aSubject = "Message from " + theFrom.toString();
        }

        while ( aLine != null ) {
            aMessage += aLine + "\n";
            aLine = aReader.readLine();
        }

        InternetAddress[] theToEmails = new InternetAddress[1];
        theToEmails[0] = theTo;

        // Ultimately we do a "deliver", allowing us to change the mechanism for
        // all mail delivery from the MailManager
        MailManager.global().deliver(theFrom, theToEmails, aSubject, aMessage, MailManager.Q_NO_THROW);
    }
}
