/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/template/TemplateParser.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: TemplateParser.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util.template;

import java.io.*;
import java.util.*;

public class TemplateParser {

    public static void main(String args[]) throws IOException {
        try {
            new TemplateParser().run(args);
        } catch (Exception e) {
            System.out.println("TemplateParser.main:" + e.toString());
            e.printStackTrace();
        }
    }
    
    private void run(String args[]) throws IOException, TemplateParserException {

        if ((args.length < 1) || (args.length > 3)) printUsage();
        
        String symbolFile = null;
        String outputFile = null;
        if (2 <= args.length)
            symbolFile = args[1];
        else
            symbolFile = args[0];
    
        if (3 <= args.length)
            outputFile = args[2];
        else
            outputFile = args[0] + ".properties";

        if (-1 == symbolFile.indexOf('.'))
            symbolFile += ".subst";
  
        TemplateParserReader    reader = null;
        PrintWriter             writer = null; 

        reader = new TemplateParserReader(args[0] + ".tplt", symbolFile, null);
        
        try {
            writer = new PrintWriter(new FileWriter(outputFile));
        } catch (Exception e) {
            System.out.println("Error opening output file " + args[0] + ".properties: " + e);
            System.exit(1);
        }
        
        int             lineNumber = 0;
        
        try {
                
            for (;;) {
                lineNumber++;
                String line = reader.readLine();
                if (null == line) break;
                writer.println(line);
            }
        } finally {
            try { if (null != reader) reader.close(); } catch (Exception e2) {}
            try { if (null != writer) writer.close(); } catch (Exception e2) {}
        }
    }
    
    private void printUsage() {
        System.out.println("java TemplateParser <template-filename> [<symbol-file>] [<output-filename>]");
        System.out.println("\ta symbol file must be specified if specifying an output file");
        System.out.println("\tif no output filename is specified, output file will be <template-filename>.properties extension");
        System.exit(1);
    }
}
